<?php
/**
*
* @package phpBB Arcade
* @version $Id: cache.php 126 2012-12-15 12:22:16Z killbill $
* @copyright (c) 2008-2011 JRSweets - jrsweets@gmail.com
* @copyright (c) 2011-2013 http://phpbbarcade.jatek-vilag.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

class arcade_cache
{
	/**
	* Obtain arcade config
	*/
	function obtain_arcade_config()
	{
		global $db, $cache;

		if (($config = $cache->get('_arcade')) !== false)
		{
			$sql = 'SELECT config_name, config_value
					FROM ' . ARCADE_CONFIG_TABLE . '
					WHERE is_dynamic = 1';
			$result = $db->sql_query($sql);

			while ($row = $db->sql_fetchrow($result))
			{
				$config[$row['config_name']] = $row['config_value'];
			}
			$db->sql_freeresult($result);
		}
		else
		{
			$config = $cached_config = array();

			$sql = 'SELECT config_name, config_value, is_dynamic
					FROM ' . ARCADE_CONFIG_TABLE;
			$result = $db->sql_query($sql);

			while ($row = $db->sql_fetchrow($result))
			{
				if (!$row['is_dynamic'])
				{
					$cached_config[$row['config_name']] = $row['config_value'];
				}

				$config[$row['config_name']] = $row['config_value'];
			}
			$db->sql_freeresult($result);

			$cache->put('_arcade', $cached_config);
		}

		return $config;
	}

	/**
	* Obtain arcade cats
	*/
	function obtain_arcade_cats()
	{
		global $db, $cache;

		if (($cats = $cache->get('_arcade_cats')) === false)
		{
			$sql = 'SELECT cat_id, cat_name, cat_style, cat_status, cat_link, cat_type, cat_test, cat_age, cat_download, cat_password, cat_rules, cat_rules_link, parent_id, cat_parents, left_id, right_id, cat_desc, cat_desc_options, cat_desc_uid, cat_desc_bitfield, cat_rules_options, cat_rules_uid, cat_rules_bitfield, cat_games_per_page, cat_use_jackpot
					FROM ' . ARCADE_CATS_TABLE . '
					ORDER BY cat_id ASC';
			$result = $db->sql_query($sql);

			$cats = array();
			while ($row = $db->sql_fetchrow($result))
			{
				$cats[$row['cat_id']] = array(
					'cat_id'				=> $row['cat_id'],
					'cat_name'				=> $row['cat_name'],
					'cat_style'				=> $row['cat_style'],
					'cat_status'			=> $row['cat_status'],
					'cat_link'				=> $row['cat_link'],
					'cat_type'				=> $row['cat_type'],
					'cat_test'				=> $row['cat_test'],
					'cat_age'				=> $row['cat_age'],
					'cat_download'			=> $row['cat_download'],
					'cat_password'			=> $row['cat_password'],
					'cat_rules'				=> $row['cat_rules'],
					'cat_rules_link'		=> $row['cat_rules_link'],
					'cat_rules_options'		=> $row['cat_rules_options'],
					'cat_rules_uid'			=> $row['cat_rules_uid'],
					'cat_rules_bitfield'	=> $row['cat_rules_bitfield'],
					'parent_id'				=> $row['parent_id'],
					'cat_parents'			=> $row['cat_parents'],
					'left_id'				=> $row['left_id'],
					'right_id'				=> $row['right_id'],
					'cat_desc'				=> $row['cat_desc'],
					'cat_desc_options'		=> $row['cat_desc_options'],
					'cat_desc_uid'			=> $row['cat_desc_uid'],
					'cat_desc_bitfield'		=> $row['cat_desc_bitfield'],
					'cat_games_per_page'	=> $row['cat_games_per_page'],
					'cat_use_jackpot'		=> $row['cat_use_jackpot']
				);
			}
			$db->sql_freeresult($result);

			$cache->put('_arcade_cats', $cats);
		}

		return $cats;
	}

	function obtain_arcade_leaders($limit, $cat_id = false)
	{
		global $db, $cache;

		if ($cat_id || ($row = $cache->get('_arcade_leaders')) === false)
		{
			$cat_stats = ($cat_id) ? " AND g.cat_id = $cat_id" : '';

			$sql_array = array(
				'SELECT'	=> 'COUNT(g.game_id) AS total_wins, u.user_id, u.username, u.user_colour, u.user_avatar, u.user_avatar_type',
				'FROM'		=> array(
					ARCADE_GAMES_TABLE	=> 'g',
				),
				'LEFT_JOIN'	=> array(
					array(
						'FROM'	=> array(USERS_TABLE => 'u'),
						'ON'	=> 'g.game_highuser = u.user_id'
					),
				),
				'WHERE'		=> 'u.user_type IN (' . USER_NORMAL . ', ' . USER_FOUNDER . ')' . $cat_stats,
				'GROUP_BY'	=> 'u.user_id, u.username, u.user_colour, u.user_avatar, u.user_avatar_type',
				'ORDER_BY'	=> 'total_wins DESC',
			);

			$sql = $db->sql_build_query('SELECT', $sql_array);
			$result = $db->sql_query_limit($sql, $limit);
			$row = $db->sql_fetchrowset($result);
			$db->sql_freeresult($result);

			if (!$cat_id)
			{
				$cache->put('_arcade_leaders', $row);
			}
		}
		return $row;
	}

	function obtain_challenge_leaders($limit)
	{
		global $db, $cache;

		if (($row = $cache->get('_arcade_challenge_leaders')) === false)
		{
			$sql_array = array(
				'SELECT'	=> 'COUNT(c.champ_winner) AS total_wins, u.user_id, u.username, u.user_colour, u.user_avatar, u.user_avatar_type',
				'FROM'		=> array(
					ARCADE_CHALLENGE_CHAMP_TABLE	=> 'c',
				),
				'LEFT_JOIN'	=> array(
					array(
						'FROM'	=> array(USERS_TABLE => 'u'),
						'ON'	=> 'c.champ_winner = u.user_id'
					),
				),
				'WHERE'		=> 'u.user_type IN (' . USER_NORMAL . ', ' . USER_FOUNDER . ')
					AND c.champ_close = ' . CHALLENGE_CLOSE . '
					AND c.champ_winner <> ' . CHALLENGE_TIE,
				'GROUP_BY'	=> 'u.user_id, u.username, u.user_colour, u.user_avatar, u.user_avatar_type',
				'ORDER_BY'	=> 'total_wins DESC',
			);

			$sql = $db->sql_build_query('SELECT', $sql_array);
			$result = $db->sql_query_limit($sql, $limit);
			$row = $db->sql_fetchrowset($result);
			$db->sql_freeresult($result);

			$cache->put('_arcade_challenge_leaders', $row);
		}
		return $row;
	}

	function obtain_arcade_leaders_all()
	{
		global $db, $cache;

		if (($highscore_data = $cache->get('_arcade_leaders_all')) === false)
		{
			$highscore_data = array();
			$sql_array = array(
				'SELECT'	=> 'COUNT(g.game_id) AS total_wins, u.user_id',
				'FROM'		=> array(
					ARCADE_GAMES_TABLE	=> 'g',
				),
				'LEFT_JOIN'	=> array(
					array(
						'FROM'	=> array(USERS_TABLE => 'u'),
						'ON'	=> 'g.game_highuser = u.user_id'
					),
				),
				'WHERE'		=> 'u.user_type IN (' . USER_NORMAL . ', ' . USER_FOUNDER . ')',
				'GROUP_BY'	=> 'u.user_id',
				'ORDER_BY'	=> 'total_wins DESC',
			);

			$sql = $db->sql_build_query('SELECT', $sql_array);
			$result = $db->sql_query($sql);

			while ($row = $db->sql_fetchrow($result))
			{
				$highscore_data[$row['user_id']] = $row['total_wins'];
			}
			$db->sql_freeresult($result);

			$cache->put('_arcade_leaders_all', $highscore_data);
		}
		return $highscore_data;
	}

	function obtain_challenge_leaders_all()
	{
		global $db, $cache;

		if (($all_cleaders = $cache->get('_arcade_challenge_leaders_all')) === false)
		{
			$all_cleaders = array();
			$sql_array = array(
				'SELECT'	=> 'COUNT(c.champ_winner) AS total_wins, u.user_id',
				'FROM'		=> array(
					ARCADE_CHALLENGE_CHAMP_TABLE	=> 'c',
				),
				'LEFT_JOIN'	=> array(
					array(
						'FROM'	=> array(USERS_TABLE => 'u'),
						'ON'	=> 'c.champ_winner = u.user_id'
					),
				),
				'WHERE'		=> 'u.user_type IN (' . USER_NORMAL . ', ' . USER_FOUNDER . ')
					AND c.champ_close = ' . CHALLENGE_CLOSE . '
					AND c.champ_winner <> ' . CHALLENGE_TIE,
				'GROUP_BY'	=> 'u.user_id',
				'ORDER_BY'	=> 'total_wins DESC',
			);

			$sql = $db->sql_build_query('SELECT', $sql_array);
			$result = $db->sql_query($sql);

			while ($row = $db->sql_fetchrow($result))
			{
				$all_cleaders[$row['user_id']] = $row['total_wins'];
			}
			$db->sql_freeresult($result);

			$cache->put('_arcade_challenge_leaders_all', $all_cleaders);
		}

		return $all_cleaders;
	}

	function obtain_arcade_games_filesize()
	{
		global $db, $cache;

		if (($games_filesize = $cache->get('_arcade_games_filesize')) === false)
		{
			$sql = 'SELECT SUM(game_filesize) as games_filesize
					FROM ' . ARCADE_GAMES_TABLE;
			$result = $db->sql_query($sql);
			$games_filesize = (int) $db->sql_fetchfield('games_filesize');
			$db->sql_freeresult($result);

			$cache->put('_arcade_games_filesize', $games_filesize);
		}

		return $games_filesize;
	}

	function obtain_arcade_recent_sites($download_url = '')
	{
		global $cache;

		if (($sites = $cache->get('_arcade_recent_sites')) === false)
		{
			$sites = array();

			if ($download_url !== '')
			{
				$sites[] = $download_url;
			}

			$cache->put('_arcade_recent_sites', $sites);
		}

		$add_site = ($download_url !== '' && !in_array($download_url, $sites)) ? true : false;

		if ($add_site)
		{
			$sites[] = $download_url;
			$cache->put('_arcade_recent_sites', $sites);
		}

		return $sites;
	}

	function obtain_arcade_ban_users()
	{
		global $db, $cache;

		if (($all_ban_users = $cache->get('_arcade_ban_users')) === false)
		{
			$all_ban_users = array();

			$sql = 'SELECT user_id, banned_ip
					FROM ' . ARCADE_USERS_BANNED_TABLE;
			$result = $db->sql_query($sql);
			while ($row = $db->sql_fetchrow($result))
			{
				$all_ban_users[$row['user_id']] = $row['banned_ip'];
			}
			$db->sql_freeresult($result);

			$cache->put('_arcade_ban_users', $all_ban_users);
		}

		return $all_ban_users;
	}

	function obtain_arcade_plugins()
	{
		global $db, $cache;

		if (($arcade_plugins = $cache->get('_arcade_plugins')) === false)
		{
			$arcade_plugins = array();

			$sql = 'SELECT plugin_id, plugin_enabled, plugin_filename, plugin_use, plugin_page, plugin_position
					FROM ' . ARCADE_PLUGINS_TABLE . '
					ORDER BY plugin_order ASC';
			$result = $db->sql_query($sql);
			while ($row = $db->sql_fetchrow($result))
			{
				$arcade_plugins[$row['plugin_id']] = array(
					'plugin_enabled'	=> $row['plugin_enabled'],
					'plugin_filename'	=> $row['plugin_filename'],
					'plugin_use'		=> $row['plugin_use'],
					'plugin_page'		=> $row['plugin_page'],
					'plugin_position'	=> $row['plugin_position']
				);
			}
			$db->sql_freeresult($result);

			$cache->put('_arcade_plugins', $arcade_plugins);
		}

		return $arcade_plugins;
	}
}

?>