<?php
/**
*
* @package phpBB Arcade
* @version $Id: class_install.php 166 2012-12-31 11:33:37Z killbill $
* @copyright (c) 2008-2011 JRSweets - jrsweets@gmail.com
* @copyright (c) 2011-2013 http://phpbbarcade.jatek-vilag.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

class arcade_install
{
	function arcade_install($return_statements = false)
	{
		global $db, $db_tools, $phpbb_root_path, $phpEx;

		if (!class_exists('phpbb_db_tools'))
		{
			include($phpbb_root_path . 'includes/db/db_tools.' . $phpEx);
		}

		$db_tools = new phpbb_db_tools($db, $return_statements);
	}

	function verify_tables($schema_data, &$error, $msg = true)
	{
		global $db, $user, $umil;

		$table_names = array_keys($schema_data);

		if (!is_array($table_names))
		{
			$table_names = array($table_names);
		}

		$rows = 0;
		$message = '';
		$sql_ary = array();
		foreach ($table_names as $table_name)
		{
			if (!$umil->table_exists($table_name))
			{
				$error[] = $table_name;

				if ($msg)
				{
					$sql		= $umil->create_table_sql($table_name, $schema_data[$table_name]);
					$rows	   += sizeof(explode("\n", $sql));
					$sql_ary[]	= $sql;
				}
			}
		}

		if ($msg)
		{
			if (sizeof($error))
			{

				$textarea	= $this->color_text(implode('<br />', $error), 'black') . '<br /><br /><textarea readonly="readonly" onclick="this.focus(); this.select();" rows="' . $rows . '" cols="40">'.htmlspecialchars(implode("\n", $sql_ary)).'</textarea>';
				$message	= $this->color_text(sprintf($user->lang['VERIFY_MISSING_TABLES'], $textarea), 'red');
			}
			else
			{
				$message = $this->color_text($user->lang['VERIFY_ALL_TABLES']);
			}
		}

		return $message;
	}

	function verify_alter_table($mode, $table_ary, &$error)
	{
		global $db, $user;

		$db->sql_return_on_error(true);
		foreach ($table_ary as $table_name => $columns)
		{
			$err = array();

			if (!is_array($columns))
			{
				$columns = array($columns);
			}

			foreach ($columns as $column)
			{
				$sql = 'SELECT ' . $column . '
						FROM ' . $table_name;
				$result = $db->sql_query($sql);
				$row = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);

				if ($db->sql_error_triggered)
				{
					$db->sql_error_triggered = false;

					if ($mode == 'current')
					{
						$err[] = $column;
					}
				}
				else
				{
					if ($mode == 'old')
					{
						$err[] = $column;
					}
				}

				unset($row);
			}

			if (sizeof($err))
			{
				$key = ($mode == 'current') ? 'VERIFY_TABLE_NOT_ALTERED' : 'VERIFY_OLD_TABLE_ALTERED';
				$error[] = $this->color_text(sprintf($user->lang[$key], $table_name, $this->color_text(implode('<br />', $err), 'black')), 'red');
			}
			else if ($mode == 'current')
			{
				$error[] = $this->color_text(sprintf($user->lang['VERIFY_TABLE_ALTERED'], $table_name));
			}
		}
		$db->sql_return_on_error(false);
	}

	function verify_config($configs, &$error, $mode = 'arcade', $msg = true)
	{
		global $user, $config, $arcade_config;

		$rows = 0;
		$message = '';
		$sql_ary = array();
		$config_ary = ($mode == 'arcade') ? $arcade_config : $config;
		$table_name = ($mode == 'arcade') ? ARCADE_CONFIG_TABLE : CONFIG_TABLE;

		foreach ($configs as $config_data)
		{
			$config_name	= array_shift($config_data);
			$config_value	= array_shift($config_data);
			$is_dynamic		= (!empty($config_data) ? 1 : 0);

			if (!isset($config_ary[$config_name]))
			{
				$error[] = $config_name;

				if ($msg)
				{
					$sql		= 'INSERT INTO ' . $table_name . ' (config_name, config_value, is_dynamic)'."\n	".'VALUES (' . "'$config_name'" . ', ' . "'$config_value'" . ', ' . "'$is_dynamic'" . ');';
					$rows	   += sizeof(explode("\n", $sql));
					$sql_ary[]	= $sql;
				}
			}
		}

		if ($msg)
		{
			if (sizeof($error))
			{
				$textarea	= $this->color_text(implode('<br />', $error), 'black') . '<br /><br /><textarea readonly="readonly" onclick="this.focus(); this.select();" rows="' . $rows . '" cols="40">'.htmlspecialchars(implode("\n", $sql_ary)).'</textarea>';
				$message	= $this->color_text(sprintf($user->lang['VERIFY_MISSING_CONFIGS'], $textarea), 'red');
			}
			else
			{
				$message = $this->color_text($user->lang['VERIFY_ALL_CONFIGS']);
			}
		}

		return $message;
	}

	function verify_permission($auth_options, &$error, $mode = 'arcade', $msg = true)
	{
		global $db, $user;

		$rows = 0;
		$message = '';
		$sql_ary = array();
		$table_name = ($mode == 'arcade') ? ACL_ARCADE_OPTIONS_TABLE : ACL_OPTIONS_TABLE;

		foreach ($auth_options as $auth_option)
		{
			$sql = "SELECT auth_option_id
					FROM $table_name
					WHERE auth_option = '" . $db->sql_escape($auth_option) . "'";
			$result = $db->sql_query($sql);
			$row = $db->sql_fetchrow($result);
			$db->sql_freeresult($result);

			if (!$row)
			{
				$error[] = $auth_option;

				if ($msg)
				{
					$global = true;
					// forum and category permissions shouldn't be set globally
					if (($mode != 'arcade' && strpos($auth_option, 'f_') === 0) || ($mode == 'arcade' && strpos($auth_option, 'c_') === 0))
					{
						$global = false;
					}

					$is_global	= ($global) ? 1 : 0;
					$is_local	= (!$global) ? 1 : 0;

					$sql		= 'INSERT INTO ' . $table_name . ' (auth_option, is_global, is_local, founder_only)'."\n	".'VALUES (' . "'$auth_option'" . ', ' . "'$is_global'" . ', ' . "'$is_local', '0'" . ');';
					$rows	   += (sizeof(explode("\n", $sql))+1);
					$sql_ary[]	= $sql;
				}
			}
		}

		if ($msg)
		{
			if (sizeof($error))
			{
				$textarea	= $this->color_text(implode('<br />', $error), 'black') . '<br /><br /><textarea readonly="readonly" onclick="this.focus(); this.select();" rows="' . $rows . '" cols="40">'.htmlspecialchars(implode("\n", $sql_ary)).'</textarea>';
				$message	= $this->color_text(sprintf($user->lang['VERIFY_MISSING_PERMISSIONS'], $textarea), 'red');
			}
			else
			{
				$message = $this->color_text($user->lang['VERIFY_ALL_PERMISSIONS']);
			}
		}

		return $message;
	}

	function drop_columns_data($schema_changes)
	{
		$drop_data = array();
		foreach ($schema_changes as $key => $data)
		{
			if ($key == 'add_columns')
			{
				foreach ($data as $table => $columns)
				{
					foreach ($columns as $column => $column_data)
					{
						$drop_data['drop_columns'][$table][] = $column;
					}
				}
			}
		}

		return $drop_data;
	}

	function file_edit_check($path, &$error, $msg = true)
	{
		global $user, $phpbb_root_path;
		global $arcade;

		$message = '';

		$installed_styles	= $this->installed_styles();
		$installed_langs	= $this->installed_languages();

		foreach ($path as $file => $value)
		{
			$files = array();
			$file = $phpbb_root_path . $file;

			if (strpos($file, '{TEMP_PATH}') !== false || strpos($file, '{THEME_PATH}') !== false)
			{
				foreach ($installed_styles as $style)
				{
					$files[] = str_replace('{TEMP_PATH}', $style['template_path'], $file);
					$files[] = str_replace('{THEME_PATH}', $style['theme_path'], $file);
				}
			}
			else if (strpos($file, '{LANG_PATH}') !== false)
			{
				foreach ($installed_langs as $lang_path)
				{
					$files[] = str_replace('{LANG_PATH}', $lang_path, $file);
				}
			}
			else
			{
				$files[] = $file;
			}

			$files = array_unique($files);

			foreach ($files as $file)
			{
				if (file_exists($file) && $content = get_file_contents($file))
				{
					foreach ($value as $edit)
					{
						if (strpos($content, $edit) === false)
						{
							$error[] = $file;
							break;
						}
					}
				}
			}
		}

		sort($error);

		if ($msg)
		{
			if (sizeof($error))
			{
				$message = $this->color_text(sprintf($user->lang['VERIFY_MISSING_FILES_EDITED'], $this->color_text($arcade->root_key(implode('<br />', $error)), 'black')), 'red');
			}
			else
			{
				$message = $this->color_text($user->lang['VERIFY_ALL_FILES_EDITED']);
			}
		}

		return $message;
	}

	function installed_styles()
	{
		global $db;

		$sql = 'SELECT t.template_path, t.bbcode_bitfield, c.theme_path, t.template_inherits_id, t.template_inherit_path
				FROM ' . STYLES_TABLE . ' s, ' . STYLES_TEMPLATE_TABLE . ' t, ' . STYLES_THEME_TABLE . ' c
				WHERE s.template_id = t.template_id
				AND s.theme_id = c.theme_id';
		$result = $db->sql_query($sql);
		$installed_styles = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$installed_styles[] = array(
				'template_path'			=> $row['template_path'],
				'bbcode_bitfield'		=> $row['bbcode_bitfield'],
				'theme_path'			=> $row['theme_path'],
				'template_inherits_id'	=> $row['template_inherits_id'],
				'template_inherit_path'	=> $row['template_inherit_path'],
			);
		}
		$db->sql_freeresult($result);

		return $installed_styles;
	}

	function installed_languages()
	{
		global $db;

		$sql = 'SELECT lang_dir FROM ' . LANG_TABLE;
		$result = $db->sql_query($sql);
		$installed_languages = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$installed_languages[] = $row['lang_dir'];
		}
		$db->sql_freeresult($result);

		return $installed_languages;
	}

	function style_bitfield($bitfield = '')
	{
		$bitfield = base64_decode($bitfield);

		if (strlen($bitfield) >= 1)
		{
			$c = $bitfield[0];

			return (bool) (ord($c) & (1 << 3));
		}
		else
		{
			return false;
		}
	}

	function color_text($text, $color = '')
	{
		if ($color == 'black')
		{
			$text = '<span style="font-weight: normal; color:#000;">' . $text . '</span>';
		}
		else
		{
			$text = '<strong style="color:#' . (($color == 'red') ? 'BC2A4D' : '228822') . '">' . $text . '</strong>';
		}

		return $text;
	}

	function def_db($version, $current_version)
	{
		global $db, $user;

		$msg  = '<p>' . $user->lang['DATABASE_TYPE'] . ' :: <strong>' . $db->sql_layer . '</strong><br />';
		$msg .= $user->lang['PREVIOUS_VERSION'] . ' :: <strong>' . $version . '</strong><br />';
		$msg .= $user->lang['UPDATED_VERSION'] . ' :: <strong>' . $current_version . '</strong><br /><br /></p>';
		$msg .= '<h1>' . $user->lang['UPDATE_DATABASE_SCHEMA'] . '</h1>';
		$msg .= '<p><br />' . $user->lang['PROGRESS'] . ' :: <br /><strong>';

		return $msg;
	}

	/**
	* Function for triggering an sql statement
	*/
	function _sql(&$msg, $sql, &$errored, &$error_ary, $echo_dot = true)
	{
		global $db;

		if (defined('DEBUG_EXTRA'))
		{
			$msg .= $sql . '<br />';
		}

		$db->sql_return_on_error(true);

		if ($sql === 'begin')
		{
			$result = $db->sql_transaction('begin');
		}
		else if ($sql === 'commit')
		{
			$result = $db->sql_transaction('commit');
		}
		else
		{
			$result = $db->sql_query($sql);
			if ($db->sql_error_triggered)
			{
				$errored = true;
				$error_ary['sql'][] = $db->sql_error_sql;
				$error_ary['error_code'][] = $db->sql_error_returned;
			}
		}

		$db->sql_return_on_error(false);

		if ($echo_dot)
		{
			$msg .= '.<br />';
		}

		return $result;
	}

	function _write_result($no_updates, $errored, $error_ary)
	{
		global $user;

		$msg = '';

		if ($no_updates)
		{
			$msg .= ' ' . $user->lang['NO_UPDATES_REQUIRED'] . '</strong></p>';
		}
		else
		{
			$msg .= ' <span class="success">' . $user->lang['DONE'] . '</span></strong><br />' . $user->lang['RESULT'] . ' :: ';

			if ($errored)
			{
				$msg .= ' <strong>' . $user->lang['SOME_QUERIES_FAILED'] . '</strong></p><ul>';
				$user_error = $this->color_text($user->lang['ERROR'], 'red');

				for ($i = 0; $i < sizeof($error_ary['sql']); $i++)
				{
					$msg .= '<li>' . $user_error . ' :: <strong>' . htmlspecialchars($error_ary['error_code'][$i]['message']) . '</strong><br />';
					$msg .= $user->lang['SQL'] . ' :: <strong>' . htmlspecialchars($error_ary['sql'][$i]) . '</strong><br /><br /></li>';
				}

				$msg .= '</ul>';
			}
			else
			{
				$msg .= '<strong>' . $user->lang['NO_ERRORS'] . '</strong></p>';
			}
		}

		return $msg;
	}

	function requirements($mode, $sub, $req, $url, $s_hidden_fields = '')
	{
		global $db, $user, $config, $template, $phpbb_root_path, $phpEx;
		global $arcade, $arcade_config, $file_functions;

		if (defined('IN_INSTALL_PHPBB_ARCADE') && $mode != 'verify')
		{
			$template->assign_vars(array(
				'TITLE'	=> $user->lang['REQUIREMENTS_' . strtoupper($mode) . '_TITLE'],
				'BODY'	=> $user->lang['REQUIREMENTS_' . strtoupper($mode) . '_EXPLAIN']
			));
		}

		$passed = array('phpbb' => true, 'arcade' => true, 'plugin' => true, 'challenge' => true, 'files' => true, 'bbcode' => true, 'mod' => true);

		if ($mode != 'verify')
		{
			// Test for correct software versions
			$template->assign_block_vars('checks', array(
				'S_LEGEND'	=> true,
				'LEGEND'	=> $user->lang['VERSION_SETTINGS']
			));

			// Test for correct phpBB version
			if (phpbb_version_compare($config['version'], $req['phpbb_version'], '<'))
			{
				$passed['phpbb'] = false;
				$result = $this->color_text($user->lang['NO'], 'red');
			}
			else
			{
				$result = $this->color_text($user->lang['YES']);
			}

			$template->assign_block_vars('checks', array(
				'TITLE'			=> sprintf($user->lang['PHPBB_VERSION'], $req['phpbb_version']),
				'TITLE_EXPLAIN'	=> sprintf($user->lang['PHPBB_VERSION_EXPLAIN'], $req['phpbb_version']),
				'RESULT'		=> $result,

				'S_EXPLAIN'		=> true,
				'S_LEGEND'		=> false
			));

			if (!defined('IN_INSTALL_PHPBB_ARCADE') || $mode == 'update')
			{
				// Test for correct phpBB Arcade version
				if (phpbb_version_compare($arcade_config['version'], $req['arcade_version'], '<'))
				{
					$passed['arcade'] = false;
					$result = $this->color_text($user->lang['NO'], 'red');
				}
				else
				{
					$result = $this->color_text($user->lang['YES']);
				}

				$template->assign_block_vars('checks', array(
					'TITLE'			=> sprintf($user->lang['PHPBB_ARCADE_VERSION'], $req['arcade_version']),
					'TITLE_EXPLAIN'	=> sprintf($user->lang['PHPBB_ARCADE_VERSION_EXPLAIN'], $req['arcade_version']),
					'RESULT'		=> $result,

					'S_EXPLAIN'		=> true,
					'S_LEGEND'		=> false
				));

				if (defined('IN_INSTALL_PHPBB_ARCADE') && !empty($arcade_config['ul_challenge_version']) && phpbb_version_compare($arcade_config['ul_challenge_version'], '1.0.RC8', '<'))
				{
					$passed['challenge'] = false;

					$template->assign_block_vars('checks', array(
						'TITLE'			=> sprintf($user->lang['PHPBB_ARCADE_CHALLENGE_VERSION'], '1.0.RC8'),
						'TITLE_EXPLAIN'	=> sprintf($user->lang['PHPBB_ARCADE_CHALLENGE_VERSION_EXP'], '1.0.RC8'),
						'RESULT'		=> $this->color_text($user->lang['NO'], 'red'),

						'S_EXPLAIN'		=> true,
						'S_LEGEND'		=> false
					));
				}

				// Test for correct phpBB Arcade Plugin version
				if ($mode == 'update' && !empty($req['plugin_version']))
				{
					if (phpbb_version_compare($req['plugin_install_version'], $req['plugin_version'], '<'))
					{
						$passed['plugin'] = false;
						$result = $this->color_text($user->lang['NO'], 'red');
					}
					else
					{
						$result = $this->color_text($user->lang['YES']);
					}

					$template->assign_block_vars('checks', array(
						'TITLE'			=> sprintf($user->lang['PHPBB_ARCADE_PLUGIN_VERSION'], $req['plugin_name'], $req['plugin_version']),
						'TITLE_EXPLAIN'	=> sprintf($user->lang['PHPBB_ARCADE_PLUGIN_VERSION_EXPLAIN'], $req['plugin_name'], $req['plugin_version']),
						'RESULT'		=> $result,

						'S_EXPLAIN'		=> true,
						'S_LEGEND'		=> false,
					));
				}
			}
		}

		if (defined('IN_INSTALL_PHPBB_ARCADE'))
		{
			$template->assign_block_vars('checks', array(
				'S_LEGEND'	=> true,
				'LEGEND'	=> $user->lang['PHP_SETTINGS']
			));

			// Check for url_fopen
			if (@ini_get('allow_url_fopen') == '1' || strtolower(@ini_get('allow_url_fopen')) == 'on')
			{
				$result = $this->color_text($user->lang['YES']);
			}
			else
			{
				$result = $this->color_text($user->lang['NO'], 'red');
			}

			$template->assign_block_vars('checks', array(
				'TITLE'			=> $user->lang['PHP_URL_FOPEN_SUPPORT'],
				'TITLE_EXPLAIN'	=> $user->lang['ARCADE_PHP_URL_FOPEN_SUPPORT_EXPLAIN'],
				'RESULT'		=> $result,

				'S_EXPLAIN'		=> true,
				'S_LEGEND'		=> false,
			));

			// Check for curl
			if (function_exists('curl_init'))
			{
				$result = $this->color_text($user->lang['YES']);
			}
			else
			{
				$result = $this->color_text($user->lang['NO'], 'red');
			}

			$template->assign_block_vars('checks', array(
				'TITLE'			=> $user->lang['PHP_CURL_SUPPORT'],
				'TITLE_EXPLAIN'	=> $user->lang['PHP_CURL_SUPPORT_EXPLAIN'],
				'RESULT'		=> $result,

				'S_EXPLAIN'		=> true,
				'S_LEGEND'		=> false,
			));
		}

		// Check permissions on files/directories we need access to
		if (sizeof($req['writable_directories']))
		{
			$template->assign_block_vars('checks', array(
				'S_LEGEND'			=> true,
				'LEGEND'			=> $user->lang['ARCADE_FILES_REQUIRED'],
				'LEGEND_EXPLAIN'	=> $user->lang['ARCADE_FILES_REQUIRED_EXPLAIN'],
			));

			umask(0);

			$c_all = array('arcade/gamedata/', 'arcade/games/', 'games/');

			foreach ($req['writable_directories'] as $dir)
			{
				$exists = $write = false;
				$path = $phpbb_root_path . $dir;
				$file_functions->append_slash($path);

				// Try to create the directory if it does not exist
				if (!file_exists($path))
				{
					@mkdir($path, 0777);
				}

				// Now really check
				if (file_exists($path) && is_dir($path))
				{
					if (in_array($dir, $c_all))
					{
						@chmod($path, 0777);
					}
					else
					{
						phpbb_chmod($path, CHMOD_READ | CHMOD_WRITE);
					}

					$exists = true;

					if (!file_exists($path . 'index.htm') && arcade_fwrite($path . 'index.htm', '', false))
					{
						$write = true;
					}

					// Now check if it is writable by storing a simple file
					if (!$write && arcade_fwrite($path . 'test_lock', '', false))
					{
						@unlink($path . 'test_lock');
						$write = true;
					}
				}

				$passed['files'] = ($exists && $write && $passed['files']) ? true : false;

				$exists = ($exists) ? $this->color_text($user->lang['FOUND']) : $this->color_text($user->lang['NOT_FOUND'], 'red');
				$write = ($write) ? ', ' . $this->color_text($user->lang['WRITABLE']) : (($exists) ? ', ' . $this->color_text($user->lang['UNWRITABLE'], 'red') : '');

				$template->assign_block_vars('checks', array(
					'TITLE'		=> $arcade->root_key($path),
					'RESULT'	=> $exists . $write,

					'S_EXPLAIN'	=> false,
					'S_LEGEND'	=> false,
				));
			}
		}

		if ($mode == 'verify' && isset($req['bbcodes']) && sizeof(array_keys($req['bbcodes'])))
		{
			// arcade use bbcode check
			$template->assign_block_vars('checks', array(
				'S_LEGEND'			=> true,
				'LEGEND'			=> $user->lang['ARCADE_BBCODE_REQUIRED'],
				'LEGEND_EXPLAIN'	=> $user->lang['ARCADE_BBCODE_REQUIRED_EXPLAIN'],
			));

			$result = '';
			$arcade_bbcode = array_keys($req['bbcodes']);

			foreach ($arcade_bbcode as $bbcode_tag)
			{
				if ($this->bbcode_exists($bbcode_tag))
				{
					$result = $this->color_text($user->lang['FOUND']);
				}
				else
				{
					$passed['bbcode'] = false;

					if (!empty($result))
					{
						$result .= '<br /><br /><hr />';
					}

					$result .= $this->color_text($user->lang['NOT_FOUND'], 'red');
					$result .= '<br />' . $this->color_text($user->lang['BBCODE_USAGE'] . ':', 'black');
					$result .= '<textarea readonly="readonly" onclick="this.focus(); this.select();" rows="2" cols="40">' . htmlspecialchars($req['bbcodes'][$bbcode_tag]['bbcode_match']) . '</textarea>';
					$result .= '<br /><br />' . $this->color_text($user->lang['HTML_REPLACEMENT'] . ':', 'black');
					$result .= '<textarea readonly="readonly" onclick="this.focus(); this.select();" rows="2" cols="40">' . htmlspecialchars($req['bbcodes'][$bbcode_tag]['bbcode_tpl']) . '</textarea>';

					$result .= '<br /><br />' . $this->color_text($user->lang['BBCODE_HELPLINE'] . ':', 'black');
					$result .= '<textarea readonly="readonly" onclick="this.focus(); this.select();" rows="2" cols="40">' . htmlspecialchars($req['bbcodes'][$bbcode_tag]['bbcode_helpline']) . '</textarea>';

					$result .= '<br /><br />' . $this->color_text($user->lang['DISPLAY_ON_POSTING'] . ': ', 'black');
					$result .= $this->color_text(($req['bbcodes'][$bbcode_tag]['display_on_posting']) ? $user->lang['YES'] : $user->lang['NO']);
				}

				$end_bbcode_tag = (substr($bbcode_tag, -1) === '=') ? substr($bbcode_tag, 0, -1) : $bbcode_tag;

				$template->assign_block_vars('checks', array(
					'TITLE'		=> '[' . $bbcode_tag . '][/' . $end_bbcode_tag . ']',
					'RESULT'	=> $result,

					'S_EXPLAIN'	=> false,
					'S_LEGEND'	=> false,
				));
			}
		}

		if ($mode != 'verify')
		{
			if (defined('IN_INSTALL_PHPBB_ARCADE'))
			{
				$params = (!in_array(false, $passed)) ? "mode=$mode&amp;sub=$sub" : "mode=$mode&amp;sub=requirements";
				$url .= ((strpos($url, "index.{$phpEx}?") !== false) ? '&amp;' : '?') . $params;
			}
			else
			{
				$url .= (in_array(false, $passed)) ? 'action=requirements' : '';
			}

			$submit = (!in_array(false, $passed)) ? $user->lang[strtoupper($sub) . '_START'] : $user->lang['INSTALL_TEST'];

			if (in_array(false, $passed))
			{
				$s_hidden_fields = str_replace(array('install', 'update'), 'requirements', $s_hidden_fields);
			}

			$template->assign_vars(array(
				'S_HIDDEN'	=> $s_hidden_fields,
				'U_ACTION'	=> $url,
				'L_SUBMIT'	=> $submit
			));
		}

		$template->assign_var('S_ARCADE_REQUIREMENTS', true);

		return $passed;
	}

	function add_bbcodes($bbcodes)
	{
		if (!is_array($bbcodes) || !sizeof($bbcodes))
		{
			return;
		}

		global $db, $cache, $phpbb_root_path, $phpEx;

		if (!class_exists('acp_bbcodes'))
		{
			include($phpbb_root_path . 'includes/acp/acp_bbcodes.' . $phpEx);
		}
		
		$acp_bbcodes = new acp_bbcodes();

		$sql = 'SELECT MAX(bbcode_id) as max_bbcode_id
				FROM ' . BBCODES_TABLE;
		$result = $db->sql_query($sql);
		$max_bbcode_id = (int) $db->sql_fetchfield('max_bbcode_id');
		$db->sql_freeresult($result);

		$next_bbcode_id = $max_bbcode_id;

		if (($max_bbcode_id + 1) <= NUM_CORE_BBCODES)
		{
			$next_bbcode_id = NUM_CORE_BBCODES;
		}

		foreach ($bbcodes as $bbcode_tag => $values)
		{
			$sql_ary = $acp_bbcodes->build_regexp($values['bbcode_match'], $values['bbcode_tpl']);

			$sql_ary = array_merge($sql_ary, array(
				'bbcode_match'			=> $values['bbcode_match'],
				'bbcode_tpl'			=> $values['bbcode_tpl'],
				'display_on_posting'	=> $values['display_on_posting'],
				'bbcode_helpline'		=> $values['bbcode_helpline'],
			));

			if ($bbcode_id = $this->bbcode_exists($bbcode_tag))
			{
				$sql = 'UPDATE ' . BBCODES_TABLE . '
						SET ' . $db->sql_build_array('UPDATE', $sql_ary) . '
						WHERE bbcode_id = ' . $bbcode_id;
				$db->sql_query($sql);
			}
			else
			{
				$next_bbcode_id++;
				$sql_ary['bbcode_id'] = $next_bbcode_id;

				$db->sql_query('INSERT INTO ' . BBCODES_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary));
			}
		}

		$cache->destroy('sql', BBCODES_TABLE);
	}

	function bbcode_exists($bbcode_tag)
	{
		global $db;

		$sql = 'SELECT bbcode_id
				FROM ' . BBCODES_TABLE . "
				WHERE bbcode_tag = '" . $db->sql_escape($bbcode_tag) . "'";
		$result = $db->sql_query($sql);
		$bbcode_id = (int) $db->sql_fetchfield('bbcode_id');
		$db->sql_freeresult($result);

		return $bbcode_id;
	}

	function table_optimize($table_names, &$db_msg, &$errored, &$error_ary)
	{
		global $db;
		global $errored, $error_ary;

		if (!is_array($table_names))
		{
			$table_names = array($table_names);
		}

		$table_names = implode(', ', $table_names);

		switch ($db->sql_layer)
		{
			case 'mysql':
			case 'mysqli':
			case 'mysql4':
				$this->_sql($db_msg, "OPTIMIZE TABLE $table_names", $errored, $error_ary);
			break;

			case 'postgresql':
				$this->_sql($db_msg, "VACUUM ANALYZE", $errored, $error_ary);
			break;
		}
	}

	function arcade_permissions_rename($old_auth_option, $new_auth_option = '')
	{
		global $db;

		if ($this->multicall(__FUNCTION__, $old_auth_option))
		{
			return;
		}

		$sql = 'UPDATE ' . ACL_ARCADE_OPTIONS_TABLE . "
				SET auth_option = '" . $db->sql_escape($new_auth_option) . "'
				WHERE auth_option = '" . $db->sql_escape($old_auth_option) . "'";
		$db->sql_query($sql);
	}
}

/**
*
* @package phpBB Arcade umil
* @Arcade umil version 1.0.5
*
* @author Nathan Guse (EXreaction) http://lithiumstudios.org
* @author David Lewis (Highway of Life) highwayoflife@gmail.com
* @package umil
* @copyright (c) 2008 phpBB Group
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

class arcade_umil extends umil
{
	var $arcade_permissions_added = false;

	/**
	* Arcade Config Exists
	*
	* This function is to check to see if a config variable exists or if it does not.
	*
	* @param string $config_name The name of the config setting you wish to check for.
	* @param bool $return_result - return the config value/default if true : default false.
	*
	* @return bool true/false if config exists
	*/
	function arcade_config_exists($config_name, $return_result = false)
	{
		global $arcade_config, $cache;

		$sql = 'SELECT *
				FROM ' . ARCADE_CONFIG_TABLE . "
				WHERE config_name = '" . $this->db->sql_escape($config_name) . "'";
		$result = $this->db->sql_query($sql);
		$row = $this->db->sql_fetchrow($result);
		$this->db->sql_freeresult($result);

		if ($row)
		{
			if (!isset($arcade_config[$config_name]))
			{
				$arcade_config[$config_name] = $row['config_value'];

				if (!$row['is_dynamic'])
				{
					$cache->destroy('_arcade');
				}
			}

			return ($return_result) ? $row : true;
		}

		// this should never happen, but if it does, we need to remove the config from the array
		if (isset($arcade_config[$config_name]))
		{
			unset($arcade_config[$config_name]);
			$cache->destroy('_arcade');
		}

		return false;
	}

	/**
	* Arcade Config Add
	*
	* This function allows you to add a config setting.
	*
	* @param string $config_name The name of the config setting you would like to add
	* @param mixed $config_value The value of the config setting
	* @param bool $is_dynamic True if it is dynamic (changes very often) and should not be stored in the cache, false if not.
	*
	* @return result
	*/
	function arcade_config_add($config_name, $config_value = '', $is_dynamic = false)
	{
		global $arcade;

		// Multicall
		if ($this->multicall(__FUNCTION__, $config_name))
		{
			return;
		}

		$this->umil_start('ARCADE_CONFIG_ADD', $config_name);

		if ($this->arcade_config_exists($config_name))
		{
			return $this->umil_end('ARCADE_CONFIG_ALREADY_EXISTS', $config_name);
		}

		$arcade->set_config($config_name, $config_value, $is_dynamic);

		return $this->umil_end();
	}

	/**
	* Arcade Config Update
	*
	* This function allows you to update an existing config setting.
	*
	* @param string $config_name The name of the config setting you would like to update
	* @param mixed $config_value The value of the config setting
	* @param bool $is_dynamic True if it is dynamic (changes very often) and should not be stored in the cache, false if not.
	*
	* @return result
	*/
	function arcade_config_update($config_name, $config_value = '', $is_dynamic = false)
	{
		global $arcade;

		// Multicall
		if ($this->multicall(__FUNCTION__, $config_name))
		{
			return;
		}

		$this->umil_start('ARCADE_CONFIG_UPDATE', $config_name);

		if (!$this->arcade_config_exists($config_name))
		{
			return $this->umil_end('ARCADE_CONFIG_NOT_EXIST', $config_name);
		}

		$arcade->set_config($config_name, $config_value, $is_dynamic);

		return $this->umil_end();
	}

	/**
	* Arcade Config Remove
	*
	* This function allows you to remove an existing config setting.
	*
	* @param string $config_name The name of the config setting you would like to remove
	*
	* @return result
	*/
	function arcade_config_remove($config_name)
	{
		global $cache, $arcade_config;

		// Multicall
		if ($this->multicall(__FUNCTION__, $config_name))
		{
			return;
		}

		$this->umil_start('ARCADE_CONFIG_REMOVE', $config_name);

		if (!$this->arcade_config_exists($config_name))
		{
			return $this->umil_end('ARCADE_CONFIG_NOT_EXIST', $config_name);
		}

		$this->db->sql_query('DELETE FROM ' . ARCADE_CONFIG_TABLE . " WHERE config_name = '" . $this->db->sql_escape($config_name) . "'");

		unset($arcade_config[$config_name]);
		$cache->destroy('_arcade');

		return $this->umil_end();
	}

	/**
	* Arcade Permission Exists
	*
	* Check if a permission (auth) setting exists
	*
	* @param string $auth_option The name of the permission (auth) option
	* @param bool $global True for checking a global permission setting, False for a local permission setting
	*
	* @return bool true if it exists, false if not
	*/
	function arcade_permission_exists($auth_option, $global = true)
	{
		// category permissions shouldn't be set globally
		if (strpos($auth_option, 'c_') === 0)
		{
			$global = false;
		}

		if ($global)
		{
			$type_sql = ' AND is_global = 1';
		}
		else
		{
			$type_sql = ' AND is_local = 1';
		}

		$sql = 'SELECT auth_option_id
				FROM ' . ACL_ARCADE_OPTIONS_TABLE . "
				WHERE auth_option = '" . $this->db->sql_escape($auth_option) . "'"
				. $type_sql;
		$result = $this->db->sql_query($sql);
		$row = $this->db->sql_fetchrow($result);
		$this->db->sql_freeresult($result);

		if ($row)
		{
			return true;
		}

		return false;
	}

	/**
	* Arcade Permission Add
	*
	* Add a permission (auth) option
	*
	* @param string $auth_option The name of the permission (auth) option
	* @param bool $global True for checking a global permission setting, False for a local permission setting
	*
	* @return result
	*/
	function arcade_permission_add($auth_option, $global = true)
	{
		// Multicall
		if ($this->multicall(__FUNCTION__, $auth_option))
		{
			return;
		}

		$this->umil_start('ARCADE_PERMISSION_ADD', $auth_option);

		// category permissions shouldn't be set globally
		if (strpos($auth_option, 'c_') === 0)
		{
			$global = false;
		}

		if ($this->arcade_permission_exists($auth_option, $global))
		{
			return $this->umil_end('ARCADE_PERMISSION_ALREADY_EXISTS', $auth_option);
		}

		// We've added arcade permissions, so set to true to notify the user.
		$this->arcade_permissions_added = true;

		if (!class_exists('auth_arcade_admin'))
		{
			global $phpbb_root_path, $phpEx;

			include($phpbb_root_path . 'includes/acp/auth.' . $phpEx);
		}
		$auth_arcade_admin = new auth_arcade_admin();

		// We have to add a check to see if the !$global (if global, local, and if local, global) permission already exists.  If it does, acl_add_option currently has a bug which would break the ACL system, so we are having a work-around here.
		if ($this->arcade_permission_exists($auth_option, !$global))
		{
			$sql_ary = array(
				'is_global'	=> 1,
				'is_local'	=> 1,
			);
			$sql = 'UPDATE ' . ACL_ARCADE_OPTIONS_TABLE . '
					SET ' . $this->db->sql_build_array('UPDATE', $sql_ary) . "
					WHERE auth_option = '" . $this->db->sql_escape($auth_option) . "'";
			$this->db->sql_query($sql);
		}
		else
		{
			if ($global)
			{
				$auth_arcade_admin->acl_add_option(array('global' => array($auth_option)));
			}
			else
			{
				$auth_arcade_admin->acl_add_option(array('local' => array($auth_option)));
			}
		}

		return $this->umil_end();
	}

	/**
	* Arcade Permission Remove
	*
	* Remove a permission (auth) option
	*
	* @param string $auth_option The name of the permission (auth) option
	* @param bool $global True for checking a global permission setting, False for a local permission setting
	*
	* @return result
	*/
	function arcade_permission_remove($auth_option, $global = true)
	{
		global $auth_arcade, $cache;

		// Multicall
		if ($this->multicall(__FUNCTION__, $auth_option))
		{
			return;
		}

		$this->umil_start('ARCADE_PERMISSION_REMOVE', $auth_option);

		// category permissions shouldn't be set globally
		if (strpos($auth_option, 'c_') === 0)
		{
			$global = false;
		}

		if (!$this->arcade_permission_exists($auth_option, $global))
		{
			return $this->umil_end('ARCADE_PERMISSION_NOT_EXIST', $auth_option);
		}

		if ($global)
		{
			$type_sql = ' AND is_global = 1';
		}
		else
		{
			$type_sql = ' AND is_local = 1';
		}
		$sql = 'SELECT auth_option_id, is_global, is_local FROM ' . ACL_ARCADE_OPTIONS_TABLE . "
				WHERE auth_option = '" . $this->db->sql_escape($auth_option) . "'" .
				$type_sql;
		$result = $this->db->sql_query($sql);
		$row = $this->db->sql_fetchrow($result);
		$this->db->sql_freeresult($result);

		$id = $row['auth_option_id'];

		// If it is a local and global permission, do not remove the row! :P
		if ($row['is_global'] && $row['is_local'])
		{
			$sql = 'UPDATE ' . ACL_ARCADE_OPTIONS_TABLE . '
					SET ' . (($global) ? 'is_global = 0' : 'is_local = 0') . '
					WHERE auth_option_id = ' . $id;
			$this->db->sql_query($sql);
		}
		else
		{
			// Delete time
			$this->db->sql_query('DELETE FROM ' . ACL_ARCADE_GROUPS_TABLE . ' WHERE auth_option_id = ' . $id);
			$this->db->sql_query('DELETE FROM ' . ACL_ARCADE_ROLES_DATA_TABLE . ' WHERE auth_option_id = ' . $id);
			$this->db->sql_query('DELETE FROM ' . ACL_ARCADE_USERS_TABLE . ' WHERE auth_option_id = ' . $id);
			$this->db->sql_query('DELETE FROM ' . ACL_ARCADE_OPTIONS_TABLE . ' WHERE auth_option_id = ' . $id);
		}

		// Purge the arcade auth cache
		$cache->destroy('_acl_arcade_options');
		$auth_arcade->acl_clear_prefetch();

		return $this->umil_end();
	}

	/**
	* Add a new arcade permission role
	*
	* @param string $role_name The new role name
	* @param sting $role_type The type (u_, m_, a_)
	*/
	function arcade_permission_role_add($role_name, $role_type = '', $role_description = '')
	{
		// Multicall
		if ($this->multicall(__FUNCTION__, $role_name))
		{
			return;
		}

		$this->umil_start('ARCADE_PERMISSION_ROLE_ADD', $role_name);

		$sql = 'SELECT role_id FROM ' . ACL_ARCADE_ROLES_TABLE . "
				WHERE role_name = '" . $this->db->sql_escape($role_name) . "'";
		$result = $this->db->sql_query($sql);
		$role_id = (int) $this->db->sql_fetchfield('role_id');
		$this->db->sql_freeresult($result);

		if ($role_id)
		{
			return $this->umil_end('ARCADE_ROLE_ALREADY_EXISTS', $old_role_name);
		}

		$sql = 'SELECT MAX(role_order) AS max FROM ' . ACL_ARCADE_ROLES_TABLE . "
				WHERE role_type = '" . $this->db->sql_escape($role_type) . "'";
		$result = $this->db->sql_query($sql);
		$role_order = (int) $this->db->sql_fetchfield('max');
		$this->db->sql_freeresult($result);
		$role_order = (!$role_order) ? 1 : $role_order + 1;

		$sql_ary = array(
			'role_name'			=> $role_name,
			'role_description'	=> $role_description,
			'role_type'			=> $role_type,
			'role_order'		=> $role_order,
		);

		$sql = 'INSERT INTO ' . ACL_ARCADE_ROLES_TABLE . ' ' . $this->db->sql_build_array('INSERT', $sql_ary);
		$this->db->sql_query($sql);

		return $this->umil_end();
	}

	/**
	* Update the name on a Arcade permission role
	*
	* @param string $old_role_name The old role name
	* @param string $new_role_name The new role name
	*/
	function permission_role_update($old_role_name, $new_role_name = '')
	{
		// Multicall
		if ($this->multicall(__FUNCTION__, $old_role_name))
		{
			return;
		}

		$this->umil_start('ARCADE_PERMISSION_ROLE_UPDATE', $old_role_name);

		$sql = 'SELECT role_id FROM ' . ACL_ARCADE_ROLES_TABLE . "
				WHERE role_name = '" . $this->db->sql_escape($old_role_name) . "'";
		$result = $this->db->sql_query($sql);
		$role_id = (int) $this->db->sql_fetchfield('role_id');
		$this->db->sql_freeresult($result);

		if (!$role_id)
		{
			return $this->umil_end('ARCADE_ROLE_NOT_EXIST', $old_role_name);
		}

		$sql = 'UPDATE ' . ACL_ARCADE_ROLES_TABLE . "
				SET role_name = '" . $this->db->sql_escape($new_role_name) . "'
				WHERE role_name = '" . $this->db->sql_escape($old_role_name) . "'";
		$this->db->sql_query($sql);

		return $this->umil_end();
	}

	/**
	* Remove a arcade permission role
	*
	* @param string $role_name The role name to remove
	*/
	function arcade_permission_role_remove($role_name)
	{
		global $auth_arcade;

		// Multicall
		if ($this->multicall(__FUNCTION__, $role_name))
		{
			return;
		}

		$this->umil_start('ARCADE_PERMISSION_ROLE_REMOVE', $role_name);

		$sql = 'SELECT role_id FROM ' . ACL_ARCADE_ROLES_TABLE . "
				WHERE role_name = '" . $this->db->sql_escape($role_name) . "'";
		$result = $this->db->sql_query($sql);
		$role_id = (int) $this->db->sql_fetchfield('role_id');
		$this->db->sql_freeresult($result);

		if (!$role_id)
		{
			return $this->umil_end('ARCADE_ROLE_NOT_EXIST', $role_name);
		}

		$this->db->sql_query('DELETE FROM ' . ACL_ARCADE_ROLES_DATA_TABLE . ' WHERE role_id = ' . $role_id);
		$this->db->sql_query('DELETE FROM ' . ACL_ARCADE_ROLES_TABLE . ' WHERE role_id = ' . $role_id);

		$auth_arcade->acl_clear_prefetch();

		return $this->umil_end();
	}

	/**
	* Arcade Permission Set
	*
	* Allows you to set permissions for a certain group/role
	*
	* @param string $name The name of the role/group
	* @param string|array $auth_option The auth_option or array of auth_options you would like to set
	* @param string $type The type (role|group)
	* @param bool $has_permission True if you want to give them permission, false if you want to deny them permission
	*/
	function arcade_permission_set($name, $auth_option = array(), $type = 'role', $has_permission = true)
	{
		global $auth_arcade;

		// Multicall
		if ($this->multicall(__FUNCTION__, $name))
		{
			return;
		}

		if (!is_array($auth_option))
		{
			$auth_option = array($auth_option);
		}

		$new_auth = array();
		$sql = 'SELECT auth_option_id FROM ' . ACL_ARCADE_OPTIONS_TABLE . '
				WHERE ' . $this->db->sql_in_set('auth_option', $auth_option);
		$result = $this->db->sql_query($sql);
		while ($row = $this->db->sql_fetchrow($result))
		{
			$new_auth[] = $row['auth_option_id'];
		}
		$this->db->sql_freeresult($result);

		if (!sizeof($new_auth))
		{
			return false;
		}

		$current_auth = array();

		$type = (string) $type; // Prevent PHP bug.

		switch ($type)
		{
			case 'role' :
				$this->umil_start('ARCADE_PERMISSION_SET_ROLE', $name);

				$sql = 'SELECT role_id FROM ' . ACL_ARCADE_ROLES_TABLE . "
						WHERE role_name = '" . $this->db->sql_escape($name) . "'";
				$result = $this->db->sql_query($sql);
				$role_id = (int) $this->db->sql_fetchfield('role_id');
				$this->db->sql_freeresult($result);

				if (!$role_id)
				{
					return $this->umil_end('ARCADE_ROLE_NOT_EXIST');
				}

				$sql = 'SELECT auth_option_id, auth_setting FROM ' . ACL_ARCADE_ROLES_DATA_TABLE . '
						WHERE role_id = ' . $role_id;
				$result = $this->db->sql_query($sql);
				while ($row = $this->db->sql_fetchrow($result))
				{
					$current_auth[$row['auth_option_id']] = $row['auth_setting'];
				}
				$this->db->sql_freeresult($result);
			break;

			case 'group' :
				$sql = 'SELECT group_id
						FROM ' . GROUPS_TABLE . "
						WHERE group_name = '" . $this->db->sql_escape($name) . "'";
				$result = $this->db->sql_query($sql);
				$group_id = (int) $this->db->sql_fetchfield('group_id');
				$this->db->sql_freeresult($result);

				if (!$group_id)
				{
					$this->umil_start('ARCADE_PERMISSION_SET_GROUP', $name);
					return $this->umil_end('GROUP_NOT_EXIST');
				}

				// If the group has a role set for them we will add the requested permissions to that role.
				$sql = 'SELECT auth_role_id FROM ' . ACL_ARCADE_GROUPS_TABLE . '
						WHERE group_id = ' . $group_id . '
						AND auth_role_id <> 0
						AND forum_id = 0';
				$result = $this->db->sql_query($sql);
				$role_id = (int) $this->db->sql_fetchfield('auth_role_id');
				$this->db->sql_freeresult($result);

				if ($role_id)
				{
					$sql = 'SELECT role_name FROM ' . ACL_ARCADE_ROLES_TABLE . '
							WHERE role_id = ' . $role_id;
					$result = $this->db->sql_query($sql);
					$role_name = $this->db->sql_fetchfield('role_name');
					$this->db->sql_freeresult($result);

					return $this->arcade_permission_set($role_name, $auth_option, 'role', $has_permission);
				}

				$this->umil_start('ARCADE_PERMISSION_SET_GROUP', $name);

				$sql = 'SELECT auth_option_id, auth_setting FROM ' . ACL_ARCADE_GROUPS_TABLE . '
						WHERE group_id = ' . $group_id;
				$result = $this->db->sql_query($sql);
				while ($row = $this->db->sql_fetchrow($result))
				{
					$current_auth[$row['auth_option_id']] = $row['auth_setting'];
				}
				$this->db->sql_freeresult($result);
			break;
		}

		$sql_ary = array();
		switch ($type)
		{
			case 'role' :
				foreach ($new_auth as $auth_option_id)
				{
					if (!isset($current_auth[$auth_option_id]))
					{
						$sql_ary[] = array(
							'role_id'			=> $role_id,
							'auth_option_id'	=> $auth_option_id,
							'auth_setting'		=> $has_permission,
						);
					}
				}

				$this->db->sql_multi_insert(ACL_ARCADE_ROLES_DATA_TABLE, $sql_ary);
			break;

			case 'group' :
				foreach ($new_auth as $auth_option_id)
				{
					if (!isset($current_auth[$auth_option_id]))
					{
						$sql_ary[] = array(
							'group_id'			=> $group_id,
							'auth_option_id'	=> $auth_option_id,
							'auth_setting'		=> $has_permission,
						);
					}
				}

				$this->db->sql_multi_insert(ACL_ARCADE_GROUPS_TABLE, $sql_ary);
			break;
		}

		$auth_arcade->acl_clear_prefetch();

		return $this->umil_end();
	}

	/**
	* Arcade Permission Unset
	*
	* Allows you to unset (remove) permissions for a certain group/role
	*
	* @param string $name The name of the role/group
	* @param string|array $auth_option The auth_option or array of auth_options you would like to set
	* @param string $type The type (role|group)
	*/
	function arcade_permission_unset($name, $auth_option = array(), $type = 'role')
	{
		global $auth_arcade;

		// Multicall
		if ($this->multicall(__FUNCTION__, $name))
		{
			return;
		}

		if (!is_array($auth_option))
		{
			$auth_option = array($auth_option);
		}

		$to_remove = array();
		$sql = 'SELECT auth_option_id FROM ' . ACL_ARCADE_OPTIONS_TABLE . '
				WHERE ' . $this->db->sql_in_set('auth_option', $auth_option);
		$result = $this->db->sql_query($sql);
		while ($row = $this->db->sql_fetchrow($result))
		{
			$to_remove[] = $row['auth_option_id'];
		}
		$this->db->sql_freeresult($result);

		if (!sizeof($to_remove))
		{
			return false;
		}

		$type = (string) $type; // Prevent PHP bug.

		switch ($type)
		{
			case 'role' :
				$this->umil_start('ARCADE_PERMISSION_UNSET_ROLE', $name);

				$sql = 'SELECT role_id FROM ' . ACL_ARCADE_ROLES_TABLE . "
						WHERE role_name = '" . $this->db->sql_escape($name) . "'";
				$result = $this->db->sql_query($sql);
				$role_id = (int) $this->db->sql_fetchfield('role_id');
				$this->db->sql_freeresult($result);

				if (!$role_id)
				{
					return $this->umil_end('ARCADE_ROLE_NOT_EXIST');
				}

				$this->db->sql_query('DELETE FROM ' . ACL_ARCADE_ROLES_DATA_TABLE . ' WHERE ' . $this->db->sql_in_set('auth_option_id', $to_remove));
			break;

			case 'group' :
				$sql = 'SELECT group_id
						FROM ' . GROUPS_TABLE . "
						WHERE group_name = '" . $this->db->sql_escape($name) . "'";
				$result = $this->db->sql_query($sql);
				$group_id = (int) $this->db->sql_fetchfield('group_id');
				$this->db->sql_freeresult($result);

				if (!$group_id)
				{
					$this->umil_start('ARCADE_PERMISSION_UNSET_GROUP', $name);
					return $this->umil_end('GROUP_NOT_EXIST');
				}

				// If the group has a role set for them we will remove the requested permissions from that role.
				$sql = 'SELECT auth_role_id FROM ' . ACL_ARCADE_GROUPS_TABLE . '
						WHERE group_id = ' . $group_id . '
						AND auth_role_id <> 0';
				$result = $this->db->sql_query($sql);
				$role_id = (int) $this->db->sql_fetchfield('auth_role_id');
				$this->db->sql_freeresult($result);

				if ($role_id)
				{
					$sql = 'SELECT role_name FROM ' . ACL_ARCADE_ROLES_TABLE . '
							WHERE role_id = ' . $role_id;
					$result = $this->db->sql_query($sql);
					$role_name = $this->db->sql_fetchfield('role_name');
					$this->db->sql_freeresult($result);

					return $this->arcade_permission_unset($role_name, $auth_option, 'role');
				}

				$this->umil_start('ARCADE_PERMISSION_UNSET_GROUP', $name);

				$this->db->sql_query('DELETE FROM ' . ACL_ARCADE_GROUPS_TABLE . ' WHERE ' . $this->db->sql_in_set('auth_option_id', $to_remove));
			break;
		}

		$auth_arcade->acl_clear_prefetch();

		return $this->umil_end();
	}
}

?>