<?php
/**
*
* @package phpBB Arcade
* @version $Id: play.php 175 2013-01-05 00:59:56Z killbill $
* @copyright (c) 2008-2011 JRSweets - jrsweets@gmail.com
* @copyright (c) 2011-2013 http://phpbbarcade.jatek-vilag.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB') || !defined('IN_PHPBB_ARCADE'))
{
	exit;
}

// Grab the data for game and navlinks
$sql_array = array(
	'SELECT'	=> 'g.game_id, g.game_name, g.game_desc, g.game_control, g.game_control_desc, g.game_image, g.game_swf, g.game_width, g.game_height, g.game_scorevar, g.game_scoretype, g.game_type, game_download, g.game_filesize, g.game_votetotal, g.game_votesum, g.game_cost, g.game_reward, g.game_jackpot, g.game_use_jackpot, g.game_plays, g.game_download_cost,
					r.game_rating,
					c.*',
	'FROM'		=> array(
		ARCADE_GAMES_TABLE	=> 'g',
	),
	'LEFT_JOIN'	=> array(
		array(
			'FROM'	=> array(ARCADE_RATING_TABLE => 'r'),
			'ON'	=> 'g.game_id = r.game_id AND r.user_id = ' . (int) $user->data['user_id']
		),
		array(
			'FROM'	=> array(ARCADE_CATS_TABLE => 'c'),
			'ON'	=> 'g.cat_id = c.cat_id'
		),
	),
	'WHERE'		=> 'g.game_id = ' . (int) $game_id,
);

// This is only needed if we are playing in a popup
if ($popup)
{
	$sql_array['SELECT'] .= ', g.game_highuser, g.game_highscore, u.username';
	$sql_array['LEFT_JOIN'][] = array('FROM' => array(USERS_TABLE => 'u'), 'ON' => 'g.game_highuser = u.user_id');
}

$sql = $db->sql_build_query('SELECT', $sql_array);
$result = $db->sql_query($sql);
$game_data = $db->sql_fetchrow($result);
$db->sql_freeresult($result);

if (!$game_data)
{
	trigger_error($user->lang['NO_GAME_ID'] . $arcade->gbl($popup));
}

if (!$auth_arcade->acl_get('c_play', $game_data['cat_id']))
{
	trigger_error($user->lang['NO_PERMISSION_ARCADE_PLAY'] . $arcade->gbl($popup));
}

if ($game_data['cat_status'] == ITEM_LOCKED)
{
	trigger_error($user->lang['ARCADE_CAT_LOCKED_ERROR'] . $arcade->gbl($popup));
}

// age limit verify
if ($game_data['cat_age'])
{
	$arcade->verify_age('play', $game_data['cat_age'], $arcade->gbl($popup));
}

// Check flood control
$interval = ($user->data['user_id'] == ANONYMOUS) ? $arcade_config['play_anonymous_interval'] : $arcade_config['play_interval'];
$social_rel = (!$user->data['is_registered'] && (strpos($user->data['session_browser'] , 'www.facebook.com') !== false || strpos($user->data['session_browser'] , 'NING/1.0') !== false || strpos($user->data['session_browser'] , 'topsy.com') !== false)) ? true : false;

if (!$social_rel)
{
	if ($interval && !$auth_arcade->acl_get('c_ignoreflood_play', $game_data['cat_id']))
	{
		$time = time() - $interval;
		if ($user->data['user_arcade_last_play'] > $time)
		{
			trigger_error(sprintf($user->lang['ARCADE_NO_PLAY_TIME'], $arcade->time_format($interval, true), $arcade->time_format($user->data['user_arcade_last_play'] - $time, true)) . $arcade->gbl($popup));
		}
	}

	// Check if the limt play is set in ACP, except for guest, bot users.
	if (!$auth_arcade->acl_get('c_ignorecontrol', $game_data['cat_id']))
	{
		$arcade->limit_play($arcade->gbl($popup));
	}
}

$game_width = $game_height = 0;
$arcade->set_game_size($game_width, $game_height, $game_data['game_width'], $game_data['game_height'], $game_data['game_swf']);
$game_file = $file_functions->remove_extension($game_data['game_swf']);
$game_path = $arcade->set_path($game_data['game_swf']);
$game_swf = ($arcade->get_protection($game_data['game_type'])) ? $arcade->url("swf=" . $game_file) : $game_path;

if (!file_exists($game_path))
{
	trigger_error(sprintf($user->lang['ARCADE_GAME_FILE_NOT_FOUND'], $game_data['game_name']) . $arcade->gbl($popup));
}

unset($game_path);

$page_title = $user->lang['ARCADE'] . ' - ' . $game_data['cat_name'] . ' - ' . $game_data['game_name'];

// Category is passworded ... check whether access has been granted to this
// user this session, if not show login box before playing the game
if ($game_data['cat_password'])
{
	login_arcade_box($game_data);
}

if ($game_data['cat_test'])
{
	$template->assign_var('S_ARCADE_CAT_TEST_MODE', true);
}

$challenge_enabled = (!$game_data['cat_test'] && $arcade->access('challenge', false)) ? true : false;

if ($challenge_enabled)
{
	$arcade->challenge_check($game_data, $popup);
}

$purge_games_cache = false;

// Handle points if needed and if we are not in a test category
if ($arcade->points['show'] && !$game_data['cat_test'])
{
	$game_cost = $arcade->get_cost($game_data);
	$game_reward = $arcade->get_reward($game_data);
	$game_download_cost = $arcade->get_download_cost($game_data);

	if ($game_cost != ARCADE_FREE && $game_cost > 0 && !$auth_arcade->acl_get('c_playfree', $game_data['cat_id']))
	{
		if (!$arcade->set_points('subtract', $user->data['user_id'], $game_cost))
		{
			trigger_error(sprintf($user->lang['ARCADE_NO_POINTS'], $arcade->points['name']) . $arcade->gbl($popup));
		}

		// The jackpot is only increased if the user actually contributed something to play
		if ($arcade->use_jackpot($game_data))
		{
			$purge_games_cache = true;
			$game_reward = $arcade->set_jackpot('add', $game_data);
		}
	}
}

// We will set this cookie to let the arcade know that the user is playing the game
$gidencoded = $arcade->session->game(true, $game_data['game_id'], $game_data['game_type'], $popup);

if (!$gidencoded)
{
	$lang_ary = array(
		'MESSAGE_TITLE'		=> $user->lang['CONFIRM'],
		'MESSAGE_TEXT'		=> sprintf($user->lang['ARCADE_RESTART_GAME_EXPLAIN'], $game_data['game_name']),
		'CONFIRM_BUTTON1'	=> sprintf($user->lang['ARCADE_RESTART_GAME'], $game_data['game_name']),
		'PAGE_TITLE'		=> sprintf($user->lang['ARCADE_RESTART_GAME'], $game_data['game_name'])
	);

	$arcade->confirm($lang_ary, $popup, $game_data['game_id']);
}

// Increase play count for games that do not submit scores
// only if we are not in a test category, we are basically
// only counting views for non scoring games and the users
// play count and time will not be increased. Currently if
// a NOSCORE_GAME is resynced it will reset the plays back
// to zero because there will be no data inside the plays
// table to accurately resync with.
if ($game_data['game_type'] == NOSCORE_GAME && !$game_data['cat_test'])
{
	$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
			SET game_plays = game_plays + 1
			WHERE game_id = ' . (int) $game_data['game_id'];
	$db->sql_query($sql);

	$purge_games_cache = true;
}

if ($purge_games_cache)
{
	$cache->destroy('sql', ARCADE_GAMES_TABLE);
}

$use_super_champion	= false;
$game_control		= $arcade->game_control($game_data['game_control']);
$game_desc			= str_replace("\n", '', censor_text(nl2br((($arcade_config['game_desc_parse_html']) ? htmlspecialchars_decode($game_data['game_desc']) : $game_data['game_desc']))));
$game_control_desc	= str_replace("\n", '', censor_text(nl2br((($arcade_config['game_desc_parse_html']) ? htmlspecialchars_decode($game_data['game_control_desc']) : $game_data['game_control_desc']))));

switch($mode)
{
	case 'play':
		generate_arcade_nav($game_data, true);

		// Get the favorite games data for the current user...
		$game_fav_data = $arcade->get_fav_data();

		if ($arcade_config['super_champion'] && $game_data['game_type'] != NOSCORE_GAME)
		{
			if ($super_champion = $arcade->super_champion($game_data['game_id']))
			{
				$use_super_champion = true;

				$avatar_width	= (int) $arcade_config['super_champion_avatar_width'];
				$avatar_height	= (int) $arcade_config['super_champion_avatar_height'];

				if (!$avatar_width || !$avatar_height)
				{
					$avatar_width	= $super_champion['user_avatar_width'];
					$avatar_height	= $super_champion['user_avatar_height'];
				}

				$template->assign_vars(array(
					'SUPER_CHAMPION_USER_NAME'	=> get_arcade_user_name('full', $super_champion['user_id'], $super_champion['username'], $super_champion['user_colour'], 'profile', 'x'),
					'SUPER_CHAMPION_AVATAR'		=> ($arcade_config['super_champion_avatar'] && $arcade->optionget('view_avatars')) ? get_arcade_user_avatar($super_champion['user_avatar'], $super_champion['user_avatar_type'], $avatar_width, $avatar_height, 'x', 'arcade', $super_champion['user_id']) : false,
					'SUPER_CHAMPION_SCORE'		=> (!$arcade->hidden_score($game_data['game_id'], $super_champion['user_id'])) ? $arcade->number_format($super_champion['score']) : $user->lang['HIDDEN'],
					'SUPER_CHAMPION_SCORE_DATE'	=> sprintf($user->lang['ARCADE_BEST_DATE_EXPLAIN'], $user->format_date($super_champion['score_date']))
				));
			}
		}

		$template->assign_vars(array(
			'S_ARCADE_FAVORITE'			=> ($auth->acl_get('u_arcade_favorites')) ? true : false,
			'S_RESOLUTION_SELECT'		=> ($arcade_config['resolution_select'] && $auth->acl_get('u_arcade_resolution')) ? true : false,
			'S_CAN_DOWNLOAD'			=> ($game_data['cat_download'] && $game_data['game_download'] && $auth->acl_get('u_arcade_download') && $auth_arcade->acl_get('c_download', $game_data['cat_id'])) ? true : false,
			'S_CAN_REPORT'				=> ($auth_arcade->acl_get('c_report', $game_data['cat_id'])) ? true : false,
			'S_DISPLAY_DESC'			=> ($arcade_config['display_desc']) ? true : false,
			'S_DISPLAY_GAME_CONTROL'	=> ($arcade_config['display_game_control'] && ($game_data['game_control'] || $game_control_desc)) ? true : false,
			'S_DISPLAY_GAME_TYPE'		=> ($arcade_config['display_game_type']) ? true : false,
			'S_FEELING_LUCKY'			=> ($auth_arcade->acl_getc_global('c_play') && sizeof($arcade->games())) ? true : false,

			'U_ACTION'					=> $arcade->url("mode=play&amp;g={$game_data['game_id']}"),
			'U_GAME_DOWNLOAD'			=> $arcade->url("mode=download&amp;g={$game_data['game_id']}"),
			'U_GAME_REPORT'				=> $arcade->url("mode=report&amp;g={$game_data['game_id']}") . '#report_game',
			'U_GAME_EDIT'				=> ($auth->acl_get('a_arcade_game') && !empty($user->data['is_registered'])) ? append_sid("{$phpbb_root_path}adm/index.$phpEx", "i=arcade_manage&amp;mode=games&amp;action=edit&amp;g={$game_data['game_id']}", true, $user->session_id) : '',
			'U_SCORE_EDIT'				=> ($auth->acl_get('a_arcade_user') && !empty($user->data['is_registered']) && ($game_data['game_plays'] || $use_super_champion)) ? append_sid("{$phpbb_root_path}adm/index.$phpEx", "i=arcade_manage&amp;mode=users&amp;action=show_scores&amp;g={$game_data['game_id']}", true, $user->session_id) : '',

			'GAME_DESC'					=> $game_desc,
			'GAME_CONTROL'				=> $game_control,
			'GAME_CONTROL_DESC'			=> $game_control_desc,
			'GAME_TYPE'					=> arcade_game_type($game_data['game_type'], true),
			'GAME_FILESIZE'				=> ($game_data['game_filesize'] > 0 ) ? sprintf($user->lang['ARCADE_GAMES_FILESIZE'], get_formatted_filesize($game_data['game_filesize'])) : sprintf($user->lang['ARCADE_GAMES_FILESIZE'], get_formatted_filesize($arcade->set_filesize($game_data['game_id']))),
			'GAME_FAV_IMG'				=> (is_array($game_fav_data)) ? $arcade->set_fav_image($game_fav_data, $game_data['game_id']) : '',
			'GAME_RATING_IMG'			=> $arcade->set_rating_image($game_data),
			'SMALL_IMAGE'				=> $arcade->get_image('src', 'img', 'small.png'),
			'BIG_IMAGE'					=> $arcade->get_image('src', 'img', 'big.png'),
			'AUTO_IMAGE'				=> $arcade->get_image('src', 'img', 'automatic.png'),
			'FULL_SCREEN'				=> $arcade->get_image('src', 'img', 'full_screen.png'),
			'OPEN_CLOSE_IMAGE'			=> $arcade->get_image('src', 'img', 'open_close.png'),
			'L_RESTART_GAME'			=> sprintf($user->lang['ARCADE_RESTART_GAME'], $game_data['game_name'])
		));

		if ($arcade->points['show'] && !$game_data['cat_test'])
		{
			$template->assign_vars(array(
				'S_SHOW_POINTS'	=> true,
				'S_USE_JACKPOT'	=> ($arcade->use_jackpot($game_data)) ? true : false,

				'AUSER_POINTS'	=> $arcade->number_format($arcade->points['total']) . ' ' . $arcade->points['name'],
				'POINTS_NAME'	=> $arcade->points['name'],
				'GAME_COST'		=> ($game_cost == ARCADE_FREE || $auth_arcade->acl_get('c_playfree', $game_data['cat_id'])) ? $user->lang['ARCADE_FREE2'] : $arcade->number_format($game_cost) . ' ' . $arcade->points['name'],
				'GAME_REWARD'	=> ($game_reward == ARCADE_FREE) ? $user->lang['ARCADE_NONE'] : $arcade->number_format($game_reward) . ' ' . $arcade->points['name'],
				'DOWNLOAD_COST'	=> ($game_download_cost == ARCADE_FREE || $auth_arcade->acl_get('c_downloadfree', $game_data['cat_id'])) ? $user->lang['ARCADE_FREE'] : $arcade->number_format($game_download_cost) . ' ' . $arcade->points['name']
			));
		}
	break;

	case 'popup':
		$highscore = (!empty($game_data['username']) && !$arcade->hidden_score($game_data['game_id'], $game_data['game_highuser'])) ? $arcade->number_format($game_data['game_highscore']) : $user->lang['HIDDEN'];
		$popup_title = (!empty($game_data['username'])) ? sprintf($user->lang['ARCADE_POPUP_HIGHUSER'], $game_data['game_name'], $game_data['username'], $highscore) : sprintf($user->lang['ARCADE_POP_NO_HIGHSCORE'], $game_data['game_name']);

		$template->assign_var('POPUP_TITLE', $popup_title);
	break;
}

if ($game_data['game_type'] != NOSCORE_GAME)
{
	$scores_limit = intval($arcade_config['game_scores']);

	if (!$scores_limit)
	{
		$scores_limit = (int) $arcade_config['arcade_flash_highscores'];
		$template->assign_var('S_SCORE_DISABLED', true);
	}

	$lastscore = '';
	$highscores = array();
	$position = $actual_position = $i = 0;
	$score_columns = 's.score, s.score_date' . (($arcade_config['game_comment']) ? ', s.comment_text, s.comment_bitfield, s.comment_options, s.comment_uid' : '');
	$score_order = ($game_data['game_scoretype'] == SCORETYPE_HIGH) ? 'DESC' : 'ASC';

	$sql_array = array(
		'SELECT'	=> $score_columns . ', p.total_time, p.total_plays, u.user_type, u.username, u.user_id, u.user_colour, u.user_avatar_type, u.user_avatar, u.user_avatar_width, u.user_avatar_height, a.user_arcade_options',
		'FROM'		=> array(
			ARCADE_SCORES_TABLE	=> 's',
		),
		'LEFT_JOIN'	=> array(
			array(
				'FROM'	=> array(ARCADE_PLAYS_TABLE => 'p'),
				'ON'	=> 's.game_id = p.game_id AND s.user_id = p.user_id'
			),
			array(
				'FROM'	=> array(USERS_TABLE => 'u'),
				'ON'	=> 's.user_id = u.user_id'
			),
			array(
				'FROM'	=> array(ARCADE_USERS_TABLE => 'a'),
				'ON'	=> 's.user_id = a.user_id'
			),
		),
		'WHERE'		=> 's.game_id = ' . (int) $game_data['game_id'] . '
			AND s.user_id = u.user_id',
		'ORDER_BY'	=> 's.score ' . $score_order . ', s.score_date ASC',
	);

	$sql = $db->sql_build_query('SELECT', $sql_array);
	$result = $db->sql_query($sql);
	while ($score_data = $db->sql_fetchrow($result))
	{
		$actual_position++;

		if ($user->data['user_id'] != $score_data['user_id'] && $actual_position > $scores_limit)
		{
			continue;
		}

		// Handle the game champion
		if ($actual_position == 1)
		{
			if ($use_super_champion && (($super_champion['score'] == $score_data['score'] && $super_champion['user_id'] == $score_data['user_id']) || ($super_champion['score'] < $score_data['score'])))
			{
				$use_super_champion = false;
			}

			$template->assign_vars(array(
				'S_GAME_CHAMPION'		=> true,
				'S_CHALLENGE_CHAMPION'	=> ($challenge_enabled && $user->data['user_id'] != $score_data['user_id'] && $arcade->optionget('challenge', $score_data['user_arcade_options'], true) && in_array($score_data['user_type'], array(USER_NORMAL, USER_FOUNDER)) && !in_array($score_data['user_id'], array_keys($arcade->ban_users()))) ? true : false,
				'S_USER_ID'				=> $user->data['user_id'],

				'U_CHALLENGE_CHAMPION'	=> $arcade->url('mode=champion_challenge&amp;g=' . $game_data['game_id'] . '&amp;u=' . $score_data['user_id']),

				'BEST_USER_NAME'		=> get_arcade_user_name('full', $score_data['user_id'], $score_data['username'], $score_data['user_colour'], 'arcade', 'x'),
				'BEST_COMMENT'			=> ($arcade_config['game_comment'] && $score_data['comment_text'] !== '') ? generate_text_for_display($score_data['comment_text'], $score_data['comment_uid'], $score_data['comment_bitfield'], $score_data['comment_options']) : '',
				'BEST_PLAYED'			=> $arcade->number_format($score_data['total_plays']),
				'BEST_TIME'				=> $arcade->time_format($score_data['total_time']),
				'BEST_SCORE'			=> (!$arcade->hidden_score($game_data['game_id'], $score_data['user_id'])) ? $arcade->number_format($score_data['score']) : $user->lang['HIDDEN'],
				'BEST_DATE_EXPLAIN'		=> sprintf($user->lang['ARCADE_BEST_DATE_EXPLAIN'], $user->format_date($score_data['score_date'])),
				'FIRST_AVATAR'			=> ($arcade->optionget('view_avatars')) ? get_arcade_user_avatar($score_data['user_avatar'], $score_data['user_avatar_type'], $score_data['user_avatar_width'], $score_data['user_avatar_height'], 'x', 'arcade', $score_data['user_id']) : false
			));

			if ($challenge_enabled && $arcade->points['show'])
			{
				$template->assign_vars(array(
					'S_CHALLENGE_BET_FIX'	=> ($arcade_config['challenge_bet_fix']) ? true : false,

					'CHALLENGE_BET_MIN'		=> ($arcade_config['challenge_bet_Minimum']) ? sprintf($user->lang['CHALLENGE_BET_MIN'], $arcade->number_format($arcade_config['challenge_bet_Minimum']) . ' ' . $arcade->points['name']) : false,
					'CHALLENGE_BET_MAX'		=> ($arcade_config['challenge_bet_Maximum']) ? sprintf($user->lang['CHALLENGE_BET_MAX'], $arcade->number_format($arcade_config['challenge_bet_Maximum']) . ' ' . $arcade->points['name']) : false,
					'CHALLENGE_BET'			=> number_format((($arcade_config['challenge_bet_fix']) ? $arcade_config['challenge_bet_fix'] : $arcade_config['challenge_bet_Minimum']), 3)
				));
			}
		}

		if ($lastscore != $score_data['score'])
		{
			$position = $actual_position;
		}

		$lastscore = $score_data['score'];
		$username = get_arcade_user_name('full', $score_data['user_id'], $score_data['username'], $score_data['user_colour'], 'arcade', 'x');

		if ($actual_position > $scores_limit)
		{
			$i--;
		}

		$highscores[$i] = array(
			'pos'	=> $position,
			'uid'	=> $score_data['user_id'],
			'uname'	=> $username,
			'fname'	=> urlencode($username),
			'score'	=> (!$arcade->hidden_score($game_data['game_id'], $score_data['user_id'])) ? $arcade->number_format($score_data['score']) : $user->lang['HIDDEN']
		);

		// We break here because we no longer need to loop through the data, we have what we need
		if ($actual_position > $scores_limit)
		{
			break;
		}

		$i++;
	}
	$db->sql_freeresult($result);

	foreach ($highscores as $line => $row)
	{
		$template->assign_block_vars('scorerow', array(
			'LINE'				=> $line,
			'POS'				=> $row['pos'],
			'USER_ID'			=> $row['uid'],
			'FULL_USERNAME'		=> $row['uname'],
			'FLASH_USERNAME'	=> $row['fname'],
			'SCORE'				=> $row['score']
		));
	}

	if ($use_super_champion)
	{
		$template->assign_var('S_GAME_SUPER_CHAMPION', true);
	}
}

$arcade->update_last_play_time($game_data['game_id'], $game_data['cat_id']);

$template->assign_vars(array(
	'L_ARCADE_FLASH_VERSION'	=> sprintf($user->lang['ARCADE_FLASH_VERSION'], $arcade_config['flash_version']),

	'T_ARCADE_FLASH_UPDATE'		=> (file_exists($phpbb_root_path . 'arcade/swf/flash_player_update.swf')) ? $arcade->url("swf=flash_player_update&amp;p=1") : false,

	'S_ARCADE_GAME_PLAY'		=> true,
	'S_POPUP'					=> ($popup) ? true : false,
	'S_USE_HIGHSCORES'			=> ($game_data['game_type'] == NOSCORE_GAME) ? false : true,

	'FLASH_VERSION'				=> $arcade_config['flash_version'],
	'ARCADE_SWF'				=> $arcade->url("swf=arcade&p=1&v={$arcade_config['version']}"),
	'ARCADE_GAME_SWF'			=> $game_swf,
	'GAME_ID'					=> $game_data["game_id"],
	'GAME_SID'					=> $arcade->session->game_sid,
	'GAME_GID'					=> $gidencoded,
	'GAME_SCOREVAR'				=> phpbb_hash($game_data["game_scorevar"]),
	'GAME_NAME'					=> $game_data['game_name'],
	'GAME_WIDTH'				=> $game_width,
	'GAME_HEIGHT'				=> $game_height,
	'GAME_CONTROL_IMG'			=> (preg_match('/src=(["\'])(.*?)\1/', $game_control, $match) != false) ? $match[2] : '',

	'FLASH_ARCADE_LOGO_TXT'		=> urlencode($arcade_config['arcade_flash_logo']),
	'FLASH_GAME_NAME'			=> urlencode($game_data['game_name']),
	'FLASH_GAME_DESC'			=> urlencode($game_desc),
	'FLASH_GAME_CONTROL_DESC'	=> urlencode($game_control_desc),
	'FLASH_HIGHSCORES'			=> (int) $arcade_config['arcade_flash_highscores']
));

if ($mode == 'play')
{
	display_arcade_online();
}

page_header($page_title, false);

$html_body = ($mode == 'play') ? 'play_body' : 'popup_body';

$template->set_filenames(array(
	'body' => 'arcade/' . $html_body . '.html')
);

page_footer();

?>