<?php
/**
*
* @package phpBB Arcade
* @version $Id: plugin.php 176 2013-01-05 12:52:16Z killbill $
* @copyright (c) 2008-2011 JRSweets - jrsweets@gmail.com
* @copyright (c) 2011-2013 http://phpbbarcade.jatek-vilag.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

/**
* Class for handling phpBB Arcade Plugin
*/
class arcade_plugin
{
	var $data = array();

	function display()
	{
		global $arcade, $arcade_cache;

		$x = 0;
		$position_top = $position_middle = $position_bottom = $position_stat = false;

		foreach ($arcade_cache->obtain_arcade_plugins() as $plugin_id => $row)
		{
			if (!$row['plugin_enabled'] || (!defined('IN_PHPBB_ARCADE') && $row['plugin_use'] == ARCADE_PLUGIN_USE_ARCADE) || (defined('IN_PHPBB_ARCADE') && $row['plugin_use'] == ARCADE_PLUGIN_USE_BOARD))
			{
				continue;
			}

			$plugin_pages = array_filter(explode(':', $row['plugin_page']));

			$active_page = false;
			foreach ($plugin_pages as $plugin_page)
			{
				if (strpos($arcade->bn(), $plugin_page) !== false)
				{
					$active_page = true;
				}
			}

			$bpl = ($row['plugin_use'] == ARCADE_PLUGIN_USE_BOARD && (($row['plugin_page'] == ARCADE_INDEX_PAGE && $arcade->bn() == ARCADE_INDEX_PAGE)/* || ($row['plugin_page'] == ARCADE_ALL_PAGE)*/)) ? true : false;
			$apl = ($row['plugin_use'] == ARCADE_PLUGIN_USE_ARCADE && (($row['plugin_page'] == ARCADE_INDEX_PAGE && $arcade->bn() == 'arcade') || ($row['plugin_page'] == ARCADE_ALL_PAGE))) ? true : false;
			$apl = ($apl) ? true : (($row['plugin_use'] == ARCADE_PLUGIN_USE_ARCADE && strpos($arcade->bn(), 'arcade') !== false && $active_page) ? true : false);

			if (!$bpl && !$apl)
			{
				continue;
			}

			$plugin_file = $this->file_path($row['plugin_filename']);

			if (!file_exists($plugin_file))
			{
				continue;
			}

			$pclass = $this->class_name($row['plugin_filename']);

			if (!class_exists($pclass))
			{
				include($plugin_file);
			}

			if (class_exists($pclass))
			{
				if (!defined('IN_PHPBB_ARCADE') && !$x)
				{
					$arcade->begin(true, false);
				}

				$this->data = $row;

				switch ($this->data['plugin_position'])
				{
					case ARCADE_POSITION_TOP:
						$position_top = true;
					break;

					case ARCADE_POSITION_MIDDLE:
						$position_middle = true;
					break;

					case ARCADE_POSITION_BOTTOM:
						$position_bottom = true;
					break;

					case ARCADE_POSITION_STAT:
						$position_stat = true;
					break;
				}

				//$pclass::arcade_plugin_display();
				call_user_func(array($pclass, 'arcade_plugin_display'));
				$x++;

				$this->data = array();
				unset($row);
			}
		}

		$this->tpl(array(
			'S_ARCADE_PLUGIN_TOP'		=> $position_top,
			'S_ARCADE_PLUGIN_MIDDLE'	=> $position_middle,
			'S_ARCADE_PLUGIN_BOTTOM'	=> $position_bottom,
			'S_ARCADE_PLUGIN_STAT'		=> $position_stat
		), false);
	}

	function position()
	{
		return ($this->data['plugin_position'] == ARCADE_POSITION_TOP) ? 'top' : (($this->data['plugin_position'] == ARCADE_POSITION_MIDDLE) ? 'middle' : (($this->data['plugin_position'] == ARCADE_POSITION_BOTTOM) ? 'bottom' : (($this->data['plugin_position'] == ARCADE_POSITION_STAT) ? 'stat' : ARCADE_DISABLED)));
	}

	function tpl($vars, $tpl = true)
	{
		global $template;

		if (sizeof($vars))
		{
			$template->assign_vars($vars);
		}

		if ($tpl && ($position = $this->position()) !== ARCADE_DISABLED)
		{
			$template->assign_block_vars('arcade_plugins', array(
				'POSITION'	=> $position,
				'TPL'		=> ($tpl === true) ? 'arcade/plugins/' . $this->data['plugin_filename'] . '.html' : "arcade/{$tpl}.html"
			));
		}
	}

	function file_path($file_name = false, $type = '')
	{
		global $phpbb_root_path, $phpEx;

		$path = "{$phpbb_root_path}arcade/includes/plugins/";

		switch ($type)
		{
			case 'lang':
				$path = "{$phpbb_root_path}language/{LANG_PATH}/mods/arcade/plugins/";
			break;

			case 'temp':
				$path = "{$phpbb_root_path}styles/{TEMP_PATH}/template/arcade/plugins/";
			break;

			case 'acp':
				$path .= 'acp/';
			break;

			case 'info':
				$path .= 'acp/info/';
			break;

			case 'install':
				$path .= 'install/';
			break;

			case 'install_xml':
				$path		= "{$phpbb_root_path}arcade/xml/plugins/";
				$file_name .= '/install.xml';

				if (!file_exists($path . $file_name))
				{
					$path = $file_name = false;
				}
			break;
		}

		if ($file_name !== false)
		{
			$path .= $file_name . ((strrchr($file_name, '.') !== false) ? '' : '.' . $phpEx);
		}

		return $path;
	}

	function class_name($plugin_name, $type = '')
	{
		$cn = '';

		switch ($type)
		{
			case 'acp':
				$cn .= 'acp_';
			break;

			case 'info':
				$cn .= 'info_';
			break;

			case 'install':
				$cn .= 'install_';
			break;
		}

		return $cn .= "arcade_plugin_$plugin_name";
	}

	function add_lang($files, $plugin_path = true, $arcade_plugins = array())
	{
		$add_files = array();
		$plugin_lang_path = 'mods/arcade/plugins/';

		$path = ($plugin_path === true) ? $plugin_lang_path : '';

		if ($files !== false)
		{
			if (is_array($files))
			{
				foreach ($files as $file)
				{
					$file = $path . $file;

					if (!$this->lang_file_exists($file) && !in_array($file, $add_files))
					{
						continue;
					}

					$add_files[] = $file;
				}
			}
			else
			{
				$file = $path . $files;

				if ($this->lang_file_exists($file))
				{
					$add_files[] = $file;
				}
			}
		}

		if (sizeof($arcade_plugins))
		{
			foreach ($arcade_plugins as $plugin_id => $row)
			{
				$file = $plugin_lang_path . $row['plugin_filename'];

				if (!$this->lang_file_exists($file))
				{
					continue;
				}

				if (!in_array($file, $add_files))
				{
					$add_files[] = $file;
				}
			}
		}

		if (sizeof($add_files))
		{
			global $user;

			$user->add_lang($add_files);
		}

		return $add_files;
	}

	function lang_file_exists($file)
	{
		global $user, $phpEx;
		return ($file !== '' && file_exists($user->lang_path . $user->lang_name . '/' . $file . '.' . $phpEx)) ? true : false;
	}
}

?>