<?php
/**
*
* @package phpBB Arcade
* @version $Id: scoretype.php 161 2012-12-30 00:26:39Z killbill $
* @copyright (c) 2008-2011 JRSweets - jrsweets@gmail.com
* @copyright (c) 2011-2013 http://phpbbarcade.jatek-vilag.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB') || !defined('IN_PHPBB_ARCADE'))
{
	exit;
}

if (!class_exists('phpBBArcade') || !$arcade->version_check())
{
	trigger_error('This page is currently not available!');
}

// Start session management here so that we can set update session page to false
// This is done so that the view online from phpBB will show the correct location
$user->session_begin(false);
$auth->acl($user->data);
$arcade->session_begin();

$error	= $prepare_score = $undefined_score = false;
$time	= time();

if (!empty($scoretype))
{
	$arcade->session->game();
	$popup = $use_simple_header = $arcade->session->game_popup;

	switch ($scoretype)
	{
		case AMOD_GAME:
			$game_scorevar = $arcade->post_var('game_name');
			$score = $arcade->post_var('score', 'float');
			$prepare_score = true;
		break;

		case IBPRO_GAME:
		case ARCADELIB_GAME:
			$game_scorevar = $arcade->post_var('gname');
			$score = $arcade->post_var('gscore', 'float');
			$prepare_score = true;
		break;

		case V3ARCADE_GAME:
			$v3arcade = strtolower($arcade->post_var('sessdo'));
			$game_scorevar = $arcade->post_var('gamename');
			$micro_one = $arcade->post_var('microone');
			$score = $arcade->post_var('score', 'float');
			$fake_key = $arcade->post_var('fakekey');

			switch ($v3arcade)
			{
				case 'sessionstart':
					$session_id = '';

					if ($game_scorevar != '')
					{
						$sql = 'SELECT s.session_id
								FROM ' . ARCADE_SESSIONS_TABLE . ' s, ' . ARCADE_GAMES_TABLE . " g
								WHERE s.phpbb_session_id = '" . $db->sql_escape($user->session_id) . "'
								AND s.user_id = " . (int) $user->data['user_id'] . "
								AND s.game_id = g.game_id
								AND g.game_scorevar = '" . $db->sql_escape($game_scorevar) . "'";
						$result = $db->sql_query($sql);
						$session_id = $db->sql_fetchfield('s.session_id');
						$db->sql_freeresult($result);
					}

					$initbar = $game_scorevar . '|' . $session_id;
					$arcade->set_header_no_cache();
					echo '&connStatus=1&initbar='. $initbar .'&val=x';
					garbage_collection();
					exit_handler();
				break;

				case 'permrequest':
					$arcade->set_header_no_cache();
					echo '&validate=1&microone='. $score .'|'. $fake_key .'&val=x';
					garbage_collection();
					exit_handler();
				break;

				case 'burn':
					$data			= explode('|', $micro_one);
					$score			= (float) $data[0];
					$game_scorevar	= str_replace("\'", "''", htmlspecialchars(trim($data[1])));
					$session_id		= (isset($data[2])) ? $data[2] : true;
					$prepare_score	= true;
				break;
			}
		break;

		case IBPROV3_GAME:
			$arcade_do = strtolower(request_var('do', ''));

			if ($arcade_do == 'verifyscore')
			{
				$randchar = $arcade->session->add_game_randchar();
				$arcade->set_header_no_cache();
				echo '&randchar=' . $randchar['randchar1'] . '&randchar2=' . $randchar['randchar2'] . '&savescore=1&blah=OK';
				garbage_collection();
				exit_handler();
			}
			else if ($arcade_do == 'savescore' || $arcade_do == 'newscore')
			{
				$game_scorevar	= $arcade->post_var('gname');
				$score			= $arcade->post_var('gscore', 'float');
				$game_gkey		= $arcade->post_var('arcadegid', 'int');
				$game_skey		= $arcade->post_var('enscore', 'float');
				$prepare_score	= true;

				if ($arcade->post_var('gscore', 'string', true))
				{
					$undefined_score = true;
				}
			}
		break;

		case AR_GAME:
			$game_scorevar = $arcade->post_var('g', 'int');
			$score = $arcade->post_var('score', 'float');
			$prepare_score = true;
		break;

		case PHPBBARCADE_GAME:
			$arcade_scorevar = strtolower($arcade->post_var('sendscore'));

			if ($arcade_scorevar == 'verify')
			{
				$session_id	= $arcade->post_var('arcade_gsid');
				$session_id	= ($session_id) ? $session_id : '-error-';
				$randchar	= $arcade->session->add_game_randchar($session_id);
				unset($session_id);
				$arcade->set_header_no_cache();
				echo 'randchar1=' . $randchar['randchar1'] . '&randchar2=' . $randchar['randchar2'] . '&savescore=ok';

				garbage_collection();
				exit_handler();
			}
			else if ($arcade_scorevar == 'save')
			{
				$game_scorevar	= $arcade->post_var('arcade_gsvar');
				$session_id		= $arcade->post_var('arcade_gsid');
				$score			= $arcade->post_var('arcade_gscore', 'float');
				$game_gkey		= $arcade->post_var('arcade_gkey', 'int');
				$game_skey		= $arcade->post_var('arcade_skey', 'float');
				$prepare_score	= true;

				if ($arcade->post_var('arcade_gscore', 'string', true))
				{
					$undefined_score = true;
				}
			}
			else if ($arcade_scorevar == 'banned' && $user->data['is_registered'] && $arcade_config['corrupt_score_banned'] && $user->data['user_type'] != USER_FOUNDER)
			{
				$arcade->banned(ARCADE_BANNED_INVALID_FORM1);
			}
		break;

		default:
			$error = 'GAME_TYPE_NOT_IDENTIFIED';
		break;
	}
}

if ($arcade->access())
{
	if ($error)
	{
		$user->setup('mods/arcade/arcade');
		trigger_error($user->lang['ARCADE_' . $error] . $arcade->gbl($popup));
	}
	else if ($prepare_score)
	{
		/**
		* This function checks to make sure the game type is the expected type
		* and that the user and game ids exist in the sessions table.
		* It then returns the needed information in an array.
		*/
		if (!empty($game_scorevar) || $scoretype == PHPBBARCADE_GAME)
		{
			if (empty($arcade->session->game_sid) && (($scoretype != PHPBBARCADE_GAME) || ($scoretype == PHPBBARCADE_GAME && empty($session_id))))
			{
				$error = ($config['cookie_secure']) ? 'GAME_SESSION_ID_ERROR' : 'COOKIE_ERROR';
			}
			else
			{
				$score = (float) $score;

				$sql_array = array(
					'SELECT'		=> 's.start_time, s.session_id, s.randchar1, s.randchar2, s.randgid1, s.randgid2, s.game_popup, s.microtime,
										g.game_id, g.game_name, g.game_image, g.game_width, g.game_height, g.game_scorevar, g.game_highscore, g.game_highuser, g.game_highdate, g.game_scoretype, g.game_type, g.game_votetotal, g.game_votesum, g.game_cost, g.game_reward, g.game_jackpot, g.game_use_jackpot, g.cat_id,
										c.cat_name, c.cat_desc, c.cat_style, c.cat_desc_uid, c.cat_desc_bitfield, c.cat_desc_options, c.parent_id, c.left_id, c.right_id, c.cat_parents, c.cat_test, c.cat_cost, c.cat_reward, c.cat_use_jackpot,
										r.game_rating',
					'FROM'			=> array(ARCADE_SESSIONS_TABLE	=> 's',
					),
					'LEFT_JOIN'	=> array(
						array(
							'FROM'	=> array(ARCADE_GAMES_TABLE		=> 'g'),
							'ON'	=> 's.game_id = g.game_id'
						),
						array(
							'FROM'	=> array(ARCADE_CATS_TABLE		=> 'c'),
							'ON'	=> 'g.cat_id = c.cat_id'
						),
						array(
							'FROM'	=> array(ARCADE_RATING_TABLE => 'r'),
							'ON'	=> 'g.game_id = r.game_id AND r.user_id = ' . (int) $user->data['user_id']
						),
					),
					'WHERE' 		=> "s.phpbb_session_id = '" . $db->sql_escape($user->session_id) . "'
						AND s.user_id = " . (int) $user->data['user_id'] . '
						AND s.game_id = g.game_id',
				);

				if ($scoretype == PHPBBARCADE_GAME)
				{
					$sql_array['WHERE'] .= " AND s.session_id = '" . $db->sql_escape($session_id) . "'";
				}
				else if (is_int($game_scorevar))
				{
					$sql_array['WHERE'] .= ' AND g.game_id = ' . (int) $game_scorevar . "
											 AND s.session_id = '" . $db->sql_escape($arcade->session->game_sid) . "'";
				}
				else
				{
					$sql_array['WHERE'] .= " AND s.session_id = '" . $db->sql_escape($arcade->session->game_sid) . "'";

					$gsv_disabled = ($scoretype == IBPROV3_GAME && $undefined_score && $game_scorevar == 'arcade') ? true : false;

					if (!$gsv_disabled)
					{
						$sql_array['WHERE'] .= " AND g.game_scorevar = '" . $db->sql_escape($game_scorevar) . "'";
					}
				}

				$sql = $db->sql_build_query('SELECT', $sql_array);
				$result = $db->sql_query_limit($sql, 1);
				$game_data = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);

				if (!$game_data)
				{
					$error = 'BACK_BUTTON_ERROR';
				}
				else
				{
					$user->setup(false, $game_data['cat_style']);
					$arcade->begin(true, true, true, false, true);

					$game_data['current_time']	= $time;
					$total_time					= intval($game_data['current_time'] - $game_data['start_time']);
					$game_data['total_time']	= ($total_time <= 0) ? false : $total_time;

					if (!empty($session_id) && $session_id !== $game_data['session_id'])
					{
						$error = 'BACK_BUTTON_ERROR';
					}
					else
					{
						$delete_cookie = true;

						if ($scoretype == PHPBBARCADE_GAME || $scoretype == IBPROV3_GAME)
						{
							$microtime			= intval(($game_data['microtime'] + $arcade_config['score_form_token_lifetime']) - $time);
							$error_microtime	= ($microtime < 0) ? true : false;
							$error_randchar		= (!$game_data['randgid1'] || !$game_data['randgid2'] || !$game_data['randchar1'] || !$game_data['randchar2']) ? true : false;
							$decodegame_key		= (int) $game_data['game_id'] * $game_data['randgid1'] ^ $game_data['randgid2'];
							$decodescore_key	= ($scoretype == PHPBBARCADE_GAME) ? (($score * $game_data['randchar1']) + $game_data['randchar2']) : ($score * $game_data['randchar1'] ^ $game_data['randchar2']);
							$error_gid			= ($decodegame_key != $game_gkey) ? true : false;
							$error_gs			= ($decodescore_key != $game_skey) ? true : false;

							if ($game_data['microtime'] > 0)
							{
								$formtime = $time - $game_data['microtime'];
								$formtime = ($formtime == 0) ? 1 : $formtime;
							}
							else
							{
								$formtime = 0;
							}
						}

						if ($scoretype == PHPBBARCADE_GAME)
						{
							$use_simple_header	= $popup = ($game_data['game_popup']) ? true : false;
							$error_pch			= (!phpbb_check_hash($game_data['game_scorevar'], $game_scorevar)) ? true : false;
						}

						if ($game_data['game_type'] != $scoretype)
						{
							$arcade->update_game_type($scoretype, $game_data['game_id'], $game_data['game_name']);
							$error = 'TYPE_ERROR';
						}
						else if (($scoretype == IBPROV3_GAME) && ($error_microtime || $error_randchar || $error_gs || $error_gid))
						{
							$banned_level3 = ($arcade_config['score_security_level'] == SCORE_SECURITY_HIGH && $error_microtime) ? true : false;

							if ($arcade_config['corrupt_score_banned'] && $user->data['is_registered'] && $user->data['user_type'] != USER_FOUNDER && $banned_level3)
							{
								$arcade->banned(ARCADE_BANNED_INVALID_FORM2, false, false, $game_data['game_id'], $score, $formtime);
								redirect($arcade->url());
							}
							else if ($undefined_score)
							{
								$error = 'GAME_DATA_EMPTY';
							}
							else
							{
								$error = 'GAME_DATA_CORRUPT';
							}

							arcade_score_error_log($scoretype, $game_data, $score, $error_microtime, $error_gs, $error_gid);
						}
						else if ($scoretype == PHPBBARCADE_GAME && ($error_microtime || $error_pch || $error_randchar || $error_gs || $error_gid))
						{
							$banned_level1 = ($arcade_config['score_security_level'] < SCORE_SECURITY_HIGH && $error_pch) ? true : false;
							$banned_level3 = ($arcade_config['score_security_level'] == SCORE_SECURITY_HIGH && ($error_pch || $error_microtime)) ? true : false;

							if ($arcade_config['corrupt_score_banned'] && $user->data['is_registered'] && $user->data['user_type'] != USER_FOUNDER && ($banned_level1 || $banned_level3))
							{
								$banned_type = ($error_pch) ? ARCADE_BANNED_INVALID_FORM3 : ARCADE_BANNED_INVALID_FORM2;
								$arcade->banned($banned_type, false, false, $game_data['game_id'], $score, $formtime);
								redirect($arcade->url());
							}
							else if ($undefined_score)
							{
								$error = 'GAME_DATA_EMPTY';
							}
							else
							{
								$error = 'GAME_DATA_CORRUPT';
							}

							arcade_score_error_log($scoretype, $game_data, $score, $error_microtime, $error_gs, $error_gid, $error_pch);
						}
						else if (!$game_data['total_time'])
						{
							$arcade->add_log('critical', $game_data['game_id'], 'LOG_ARCADE_ERROR_PLAYING_TIME', $game_data['game_name'], $score);
							$error = 'TIME_ERROR';
						}
					}
				}
			}
		}
		else
		{
			if ($scoretype == IBPROV3_GAME && !$game_gkey && $game_skey)
			{
				$error = 'IBPROV3_TYPE_NOT_SUPPORTED';
			}
			else
			{
				$error = 'SCOREVAR_ERROR';
			}
		}

		if (!empty($game_data))
		{
			$arcade->session->delete('gid', $game_data['game_id']);
		}
		else
		{
			$user->setup('mods/arcade/arcade');
		}

		if ($error)
		{
			if (!empty($delete_cookie))
			{
				$arcade->session->set_cookie();
			}

			trigger_error($user->lang['ARCADE_' . $error] . ((!empty($game_data)) ? $arcade->return_links($game_data, true, $popup) : $arcade->gbl($popup)));
		}

		include($phpbb_root_path . 'arcade/includes/score.'.$phpEx);
	}
}

// Something went wrong lets send them to the arcade index.
redirect($arcade->url());

function arcade_score_error_log($scoretype, $game_data, $score, $error_microtime, $error_gs, $error_gid, $error_pch = false)
{
	global $user, $arcade;

	$errors = array();

	if ($error_microtime)
	{
		$errors[] = 'ARCADE_SCORE_ERR_MICRO_TIME';
	}

	$game_key_err = false;
	$score_key_err = false;

	if (!$game_data['randgid1'] || !$game_data['randgid2'] || !$game_data['randchar1'] || !$game_data['randchar2'])
	{
		if ((!$game_data['randchar1'] || !$game_data['randchar2']) && (!$game_data['randgid1'] || !$game_data['randgid2']))
		{
			$game_key_err = true;
			$score_key_err = true;
			$errors[] = 'ARCADE_SCORE_ERR_GS_GG_RANDCHAR';
		}
		else if (!$game_data['randchar1'] || !$game_data['randchar2'])
		{
			$score_key_err = true;
			$errors[] = 'ARCADE_SCORE_ERR_GS_RANDCHAR';
		}
		else
		{
			$game_key_err = true;
			$errors[] = 'ARCADE_SCORE_ERR_GG_RANDCHAR';
		}
	}

	if (!$score_key_err && $error_gs)
	{
		$errors[] = 'ARCADE_SCORE_ERR_GS';
	}

	if (!$game_key_err && $error_gid)
	{
		$errors[] = 'ARCADE_SCORE_ERR_GG';
	}

	if ($error_pch)
	{
		$errors[] = 'ARCADE_SCORE_ERR_PCH';
	}

	$arcade->add_log('critical', $game_data['game_id'], 'LOG_ARCADE_FORM_SCORE_ERROR' . ((sizeof($errors) > 1) ? 'S' : ''), $game_data['game_name'], arcade_game_type($scoretype), $arcade->number_format($score), implode(', ', $errors));
}

?>