<?php
/**
*
* @package phpBB Arcade install
* @version $Id: index.php 167 2013-01-02 14:11:03Z killbill $
* @copyright (c) 2008-2011 JRSweets - jrsweets@gmail.com
* @copyright (c) 2011-2013 http://phpbbarcade.jatek-vilag.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
define('IN_PHPBB', true);
define('IN_INSTALL', true);
define('IN_INSTALL_PHPBB_ARCADE', true);
define('IN_PHPBB_ARCADE', true);

$phpbb_root_path = (defined('PHPBB_ROOT_PATH')) ? PHPBB_ROOT_PATH : './../';
$phpEx = substr(strrchr(__FILE__, '.'), 1);

include($phpbb_root_path . 'common.' . $phpEx);

// Report all errors, except notices
error_reporting(E_ALL);

// Try to override some limits - maybe it helps some...
@set_time_limit(0);
$mem_limit = @ini_get('memory_limit');

if (!empty($mem_limit))
{
	$unit = strtolower(substr($mem_limit, -1, 1));
	$mem_limit = (int) $mem_limit;

	if ($unit == 'k')
	{
		$mem_limit = floor($mem_limit / 1024);
	}
	else if ($unit == 'g')
	{
		$mem_limit *= 1024;
	}
	else if (is_numeric($unit))
	{
		$mem_limit = floor((int) ($mem_limit . $unit) / 1048576);
	}

	$mem_limit = max(128, $mem_limit) . 'M';
}
else
{
	$mem_limit = '128M';
}

@ini_set('memory_limit', $mem_limit);

// Start session management
$user->session_begin();
$auth->acl($user->data);
$user->setup(array('install', 'acp/common', 'acp/modules', 'acp/posting', 'mods/arcade/install', 'mods/arcade/arcade', 'mods/info_acp_arcade', 'mods/info_ucp_arcade', 'mods/permissions_arcade'));

if (!$user->data['is_registered'])
{
	if ($user->data['is_bot'])
	{
		redirect($phpbb_root_path . 'index.' . $phpEx);
	}

	login_box();
}

if ($user->data['user_type'] != USER_FOUNDER)
{
	trigger_error('INST_ERR_AUTH');
}

if (!file_exists($phpbb_root_path . 'umil/umil.' . $phpEx))
{
	$message = (!empty($user->lang['ARCADE_UMIL_DOWNLOAD'])) ? $user->lang['ARCADE_UMIL_DOWNLOAD'] : 'Please download the latest UMIL (Unified MOD Install Library) from: %s';
	trigger_error(sprintf($message, '<a onclick="window.open(this.href); return false;" href="http://www.phpbb.com/mods/umil/">phpBB.com/mods/umil</a>'), E_USER_ERROR);
}

$arcade_error = array();

require($phpbb_root_path . 'install/arcade_dbal_schema.' . $phpEx);
require($phpbb_root_path . 'install/arcade_config.' . $phpEx);
include($phpbb_root_path . 'arcade/includes/arcade.' . $phpEx);

if (empty($user->lang['ARCADE_USER_NO_HIGHSCORES']) || empty($user->lang['ARCADE_UMIL_DOWNLOAD']) || empty($user->lang['ARCADE_BANNED_TYPE']) || empty($user->lang['UCP_ARCADE_POST']) || empty($user->lang['acl_c_downloadfree']['lang']))
{
	$message = (!empty($user->lang['INCOMPLETE_LANG_ROWS'])) ? $user->lang['INCOMPLETE_LANG_ROWS'] : 'Some changes to the language files are missing. Please apply them as described %sHERE%s.';
	$arcade_error[] = sprintf($message, '<a href="phpbbarcade.jatek-vilag.com/viewforum.php?f=10">', '</a>');
}
else
{
	if (!defined('ARCADE_PLUGINS_TABLE') || !function_exists('get_file_contents') || !defined('ARCADE_VERSION') || phpbb_version_compare(ARCADE_VERSION, $arcade_mod_config['version']['current'], '<>'))
	{
		$arcade_error[] = sprintf($user->lang['INCOMPLETE_FILES_UPLOAD'], $arcade_mod_config['version']['current']);
	}

	$content1 = get_file_contents($phpbb_root_path . 'common.' . $phpEx);
	$content2 = get_file_contents($phpbb_root_path . 'includes/auth.' . $phpEx);

	if (strpos($content1, 'arcade/includes/phpBBArcade.') === false || strpos($content2, '$auth_arcade->acl_clear_prefetch($user_id);') === false || !defined('ARCADE_CONFIG_TABLE'))
	{
		$arcade_error[] = $user->lang['INCOMPLETE_FILE_EDITS'];
	}

	unset($content1, $content2);
}

if (sizeof($arcade_error))
{
	trigger_error(implode('<br />', $arcade_error), E_USER_ERROR);
}

include($phpbb_root_path . 'includes/functions_module.' . $phpEx);
include($phpbb_root_path . 'includes/acp/acp_modules.' . $phpEx);
include($phpbb_root_path . 'includes/acp/auth.' . $phpEx);

if (!isset($config['arcade_install']))
{
	include($phpbb_root_path . 'arcade/includes/phpBBArcade.' . $phpEx);
	include($phpbb_root_path . 'arcade/includes/auth.' . $phpEx);
	$arcade = new phpBBArcade();
	$auth_arcade = new auth_arcade();
}

require($phpbb_root_path . 'umil/umil.' . $phpEx);
require($phpbb_root_path . 'arcade/includes/class_install.' . $phpEx);
include($phpbb_root_path . 'arcade/includes/functions_files.' . $phpEx);
include($phpbb_root_path . 'arcade/includes/auth_admin.' . $phpEx);

$template->set_custom_template('../adm/style', 'admin');
$template->assign_var('T_TEMPLATE_PATH', '../adm/style');

// Set some standard variables we want to force
$config['load_tplcompile'] = '1';
// the acp template is never stored in the database
$user->theme['template_storedb'] = false;

$mode			= request_var('mode', 'overview');
$sub			= request_var('sub', '');

$arcade_install	= new arcade_install(($mode == 'update') ? true : false);
$umil			= new arcade_umil(true);
$install		= new module();
$file_functions	= new file_functions();

$install->create('install', "index.$phpEx", $mode, $sub);
$install->load();

// Generate the page
$install->page_header();
$install->generate_navigation();

$template->set_filenames(array(
	'body' => $install->get_tpl_name())
);

$install->page_footer();

/**
* @package install
*/
class module
{
	var $id = 0;
	var $type = 'install';
	var $req = array();
	var $module_ary = array();
	var $filename;
	var $module_url = '';
	var $mode;
	var $sub;
	var $arcade_version;
	var $arcade_auth_sync = false;

	/**
	* Private methods, should not be overwritten
	*/
	function create($module_type, $module_url, $selected_mod = false, $selected_submod = false)
	{
		global $user, $db, $phpEx, $umil, $arcade_config, $arcade_mod_config;

		$arcade_config = array();

		// Check if the arcade is already installed
		if ($umil->table_exists(ARCADE_CONFIG_TABLE))
		{
			$sql = 'SELECT *
					FROM ' . ARCADE_CONFIG_TABLE;
			$result = $db->sql_query($sql);

			while ($row = $db->sql_fetchrow($result))
			{
				if ($row['config_name'] == 'version')
				{
					$this->arcade_version = $row['config_value'];
				}

				$arcade_config[$row['config_name']] = $row['config_value'];
			}
			$db->sql_freeresult($result);

			$this->arcade_auth_sync = phpbb_version_compare($this->arcade_version, '2.0.RC4', '<');
		}

		$this->req = array(
			'phpbb_version'			=> $arcade_mod_config['version']['phpbb'],
			'arcade_version'		=> $arcade_mod_config['version']['oldest'],
			'bbcodes'				=> $arcade_mod_config['bbcodes'],
			'writable_directories'	=> $arcade_mod_config['writable_directories']
		);

		$module = array();

		// Grab module information using Bart's "neat-o-module" system (tm)
		$dir = @opendir('.');

		if (!$dir)
		{
			$this->error($user->lang['NOT_INSTALL_DIR'], __LINE__, __FILE__);
		}

		$setmodules = 1;
		while (($file = readdir($dir)) !== false)
		{
			if (preg_match('#^arcade_ins_(.*?)\.' . $phpEx . '$#', $file))
			{
				include($file);
			}
		}
		closedir($dir);

		unset($setmodules);

		if (!sizeof($module))
		{
			$this->error($user->lang['NOT_MODULE'], __LINE__, __FILE__);
		}

		// Order to use and count further if modules get assigned to the same position or not having an order
		$max_module_order = 1000;

		foreach ($module as $row)
		{
			// Module order not specified or module already assigned at this position?
			if (!isset($row['module_order']) || isset($this->module_ary[$row['module_order']]))
			{
				$row['module_order'] = $max_module_order;
				$max_module_order++;
			}

			$this->module_ary[$row['module_order']]['name'] = $row['module_title'];
			$this->module_ary[$row['module_order']]['filename'] = $row['module_filename'];
			$this->module_ary[$row['module_order']]['subs'] = $row['module_subs'];
			$this->module_ary[$row['module_order']]['stages'] = $row['module_stages'];

			if (strtolower($selected_mod) == strtolower($row['module_title']))
			{
				$this->id = (int) $row['module_order'];
				$this->filename = (string) $row['module_filename'];
				$this->module_url = (string) $module_type . '/' . $module_url;
				$this->mode = (string) $selected_mod;
				// Check that the sub-mode specified is valid or set a default if not
				if (is_array($row['module_subs']))
				{
					$this->sub = strtolower((in_array(strtoupper($selected_submod), $row['module_subs'])) ? $selected_submod : $row['module_subs'][0]);
				}
				else if (is_array($row['module_stages']))
				{
					$this->sub = strtolower((in_array(strtoupper($selected_submod), $row['module_stages'])) ? $selected_submod : $row['module_stages'][0]);
				}
				else
				{
					$this->sub = '';
				}
			}
		} // END foreach
	} // END create

	/**
	* Load and run the relevant module if applicable
	*/
	function load($mode = false, $run = true)
	{
		global $phpbb_root_path, $phpEx;

		if ($run)
		{
			if (!empty($mode))
			{
				$this->mode = $mode;
			}

			$module = $this->filename;
			if (!class_exists($module))
			{
				$this->error('Module "' . htmlspecialchars($module) . '" not accessible.', __LINE__, __FILE__);
			}
			$this->module = new $module($this);

			if (method_exists($this->module, 'main'))
			{
				$this->module->main($this->mode, $this->sub);
			}
		}
	}

	/**
	* Output the standard page header
	*/
	function page_header()
	{
		if (defined('HEADER_INC'))
		{
			return;
		}

		define('HEADER_INC', true);
		global $template, $user, $stage, $phpbb_root_path;

		$template->assign_vars(array(
			'PAGE_TITLE'			=> $this->get_page_title(),
			'T_IMAGE_PATH'			=> $phpbb_root_path . 'adm/images/',

			'S_CONTENT_DIRECTION' 	=> $user->lang['DIRECTION'],
			'S_CONTENT_FLOW_BEGIN'	=> ($user->lang['DIRECTION'] == 'ltr') ? 'left' : 'right',
			'S_CONTENT_FLOW_END'	=> ($user->lang['DIRECTION'] == 'ltr') ? 'right' : 'left',
			'S_CONTENT_ENCODING' 	=> 'UTF-8',

			'S_USER_LANG'			=> $user->lang['USER_LANG'],
		));

		header('Content-type: text/html; charset=UTF-8');
		header('Cache-Control: private, no-cache="set-cookie"');
		header('Expires: 0');
		header('Pragma: no-cache');

		return;
	}

	/**
	* Output the standard page footer
	*/
	function page_footer()
	{
		global $db, $template;

		$template->display('body');

		// Close our DB connection.
		if (!empty($db) && is_object($db))
		{
			$db->sql_close();
		}

		if (function_exists('exit_handler'))
		{
			exit_handler();
		}
	}

	/**
	* Returns desired template name
	*/
	function get_tpl_name()
	{
		return $this->module->tpl_name . '.html';
	}

	/**
	* Returns the desired page title
	*/
	function get_page_title()
	{
		global $user;

		if (!isset($this->module->page_title))
		{
			return '';
		}

		return (isset($user->lang[$this->module->page_title])) ? $user->lang[$this->module->page_title] : $this->module->page_title;
	}

	/**
	* Generate the navigation tabs
	*/
	function generate_navigation()
	{
		global $user, $template, $phpEx;

		if (is_array($this->module_ary))
		{
			@ksort($this->module_ary);
			foreach ($this->module_ary as $cat_ary)
			{
				$cat = $cat_ary['name'];
				$l_cat = (!empty($user->lang['CAT_' . $cat])) ? $user->lang['CAT_' . $cat] : preg_replace('#_#', ' ', $cat);
				$cat = strtolower($cat);
				$url = $this->u_module("mode=$cat");

				if ($this->mode == $cat)
				{
					$template->assign_block_vars('t_block1', array(
						'L_TITLE'		=> $l_cat,
						'S_SELECTED'	=> true,
						'U_TITLE'		=> $url,
					));

					if (is_array($this->module_ary[$this->id]['subs']))
					{
						$subs = $this->module_ary[$this->id]['subs'];
						foreach ($subs as $option)
						{
							$l_option = (!empty($user->lang['SUB_' . $option])) ? $user->lang['SUB_' . $option] : preg_replace('#_#', ' ', $option);
							$option = strtolower($option);
							$url = $this->u_module('mode=' . $this->mode . "&amp;sub=$option");

							$template->assign_block_vars('l_block1', array(
								'L_TITLE'		=> $l_option,
								'S_SELECTED'	=> ($this->sub == $option),
								'U_TITLE'		=> $url,
							));
						}
					}

					if (is_array($this->module_ary[$this->id]['stages']))
					{
						$subs = $this->module_ary[$this->id]['stages'];
						$matched = false;
						foreach ($subs as $option)
						{
							$l_option = (!empty($user->lang['STAGE_' . $option])) ? $user->lang['STAGE_' . $option] : preg_replace('#_#', ' ', $option);
							$option = strtolower($option);
							$matched = ($this->sub == $option) ? true : $matched;

							$template->assign_block_vars('l_block2', array(
								'L_TITLE'		=> $l_option,
								'S_SELECTED'	=> ($this->sub == $option),
								'S_COMPLETE'	=> !$matched,
							));
						}
					}
				}
				else
				{
					$template->assign_block_vars('t_block1', array(
						'L_TITLE'		=> $l_cat,
						'S_SELECTED'	=> false,
						'U_TITLE'		=> $url,
					));
				}
			}
		}
	}

	/**
	* Output an error message
	* If skip is true, return and continue execution, else exit
	*/
	function error($error, $line, $file, $skip = false)
	{
		global $user, $db, $template;

		if ($skip)
		{
			$template->assign_block_vars('checks', array(
				'S_LEGEND'	=> true,
				'LEGEND'	=> $user->lang['INST_ERR'],
			));

			$template->assign_block_vars('checks', array(
				'TITLE'		=> basename($file) . ' [ ' . $line . ' ]',
				'RESULT'	=> '<b style="color:red">' . $error . '</b>',
			));

			return;
		}

		echo '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">';
		echo '<html xmlns="http://www.w3.org/1999/xhtml" dir="ltr">';
		echo '<head>';
		echo '<meta http-equiv="content-type" content="text/html; charset=utf-8" />';
		echo '<title>' . $user->lang['INST_ERR_FATAL'] . '</title>';
		echo '<link href="../adm/style/admin.css" rel="stylesheet" type="text/css" media="screen" />';
		echo '</head>';
		echo '<body id="errorpage">';
		echo '<div id="wrap">';
		echo '	<div id="page-header">';
		echo '	</div>';
		echo '	<div id="page-body">';
		echo '		<div id="acp">';
		echo '		<div class="panel">';
		echo '			<span class="corners-top"><span></span></span>';
		echo '			<div id="content">';
		echo '				<h1>' . $user->lang['INST_ERR_FATAL'] . '</h1>';
		echo '		<p>' . $user->lang['INST_ERR_FATAL'] . "</p>\n";
		echo '		<p>' . basename($file) . ' [ ' . $line . " ]</p>\n";
		echo '		<p><b>' . $error . "</b></p>\n";
		echo '			</div>';
		echo '			<span class="corners-bottom"><span></span></span>';
		echo '		</div>';
		echo '		</div>';
		echo '	</div>';
		echo '	<div id="page-footer">';
		echo '		Powered by phpBB &copy; 2000, 2002, 2005, 2007 <a href="http://www.phpbb.com/">phpBB Group</a>';
		echo '	</div>';
		echo '</div>';
		echo '</body>';
		echo '</html>';

		if (!empty($db) && is_object($db))
		{
			$db->sql_close();
		}

		exit_handler();
	}

	/**
	* Generate the relevant HTML for an input field and the associated label and explanatory text
	*/
	function input_field($name, $type, $value='', $options='')
	{
		global $user;
		$tpl_type = explode(':', $type);
		$tpl = '';

		switch ($tpl_type[0])
		{
			case 'text':
			case 'password':
				$size = (int) $tpl_type[1];
				$maxlength = (int) $tpl_type[2];

				$tpl = '<input id="' . $name . '" type="' . $tpl_type[0] . '"' . (($size) ? ' size="' . $size . '"' : '') . ' maxlength="' . (($maxlength) ? $maxlength : 255) . '" name="' . $name . '" value="' . $value . '" />';
			break;

			case 'textarea':
				$rows = (int) $tpl_type[1];
				$cols = (int) $tpl_type[2];

				$tpl = '<textarea id="' . $name . '" name="' . $name . '" rows="' . $rows . '" cols="' . $cols . '">' . $value . '</textarea>';
			break;

			case 'radio':
				$key_yes	= ($value) ? ' checked="checked" id="' . $name . '"' : '';
				$key_no		= (!$value) ? ' checked="checked" id="' . $name . '"' : '';

				$tpl_type_cond = explode('_', $tpl_type[1]);
				$type_no = ($tpl_type_cond[0] == 'disabled' || $tpl_type_cond[0] == 'enabled') ? false : true;

				$tpl_no = '<label><input type="radio" name="' . $name . '" value="0"' . $key_no . ' class="radio" /> ' . (($type_no) ? $user->lang['NO'] : $user->lang['DISABLED']) . '</label>';
				$tpl_yes = '<label><input type="radio" name="' . $name . '" value="1"' . $key_yes . ' class="radio" /> ' . (($type_no) ? $user->lang['YES'] : $user->lang['ENABLED']) . '</label>';

				$tpl = ($tpl_type_cond[0] == 'yes' || $tpl_type_cond[0] == 'enabled') ? $tpl_yes . '&nbsp;&nbsp;' . $tpl_no : $tpl_no . '&nbsp;&nbsp;' . $tpl_yes;
			break;

			case 'select':
				eval('$s_options = ' . str_replace('{VALUE}', $value, $options) . ';');
				$tpl = '<select id="' . $name . '" name="' . $name . '">' . $s_options . '</select>';
			break;

			case 'custom':
				eval('$tpl = ' . str_replace('{VALUE}', $value, $options) . ';');
			break;

			default:
			break;
		}

		return $tpl;
	}

	function file_not_exist_check($mode, $path, &$error, $other = false, $rpc = false, $sbr = false)
	{
		global $arcade, $phpbb_root_path;

		foreach ($path as $file)
		{
			// subsliver2 based file - image name replacement
			if ($rpc && $sbr)
			{
				$file = str_replace('icon', 'icon_mini', $file);
			}

			if ($other !== false)
			{
				$this->path_replace($mode, $other, $file);
			}

			$file = $phpbb_root_path . $file;

			if (!file_exists($file))
			{
				$msg = $arcade->root_key($file);

				if (!in_array($msg, $error))
				{
					$error[] = $msg;
				}
			}
		}
	}

	function file_exist_check($mode, $path, &$error, $other = false)
	{
		global $arcade, $phpbb_root_path;

		foreach ($path as $file)
		{
			if ($other !== false)
			{
				$this->path_replace($mode, $other, $file);
			}

			$file = $phpbb_root_path . $file;

			if (file_exists($file))
			{
				$msg = $arcade->root_key($file);

				if (!in_array($msg, $error))
				{
					$error[] = $msg;
				}
			}
		}
	}

	function file_edit_check($mode, $path, &$error, $other = false, $rpc = false, $sbr = false, $max = false, $orig_name = false)
	{
		global $user, $arcade, $phpbb_root_path;

		$file_names = array();

		foreach ($path as $file => $value)
		{
			// subsliver2 based file
			if ($rpc && $sbr)
			{

			}

			if ($other !== false)
			{
				// alien style edit error max 1
				if ($max && in_array($file, $file_names))
				{
					continue;
				}

				$file_names[] = $file;

				if ($orig_name)
				{
					$orig_path = $file;

					$this->path_replace($mode, $orig_name, $orig_path);

					if (file_exists($phpbb_root_path . $orig_path))
					{
						$file = $orig_path;
					}
				}

				$this->path_replace($mode, $other, $file);
			}

			$file = $phpbb_root_path . $file;

			if (file_exists($file) && $content = get_file_contents($file))
			{
				foreach ($value as $edit)
				{
					if (strpos($content, $edit) === false)
					{
						if ($other !== false && $max)
						{
							$msg = $arcade->root_key($file);
						}
						else
						{
							$msg = $arcade->root_key($file) . '<br /><textarea onclick="this.focus(); this.select();" rows="8" cols="40">'.htmlspecialchars($edit).'</textarea>';
						}

						if (!in_array($msg, $error))
						{
							$error[] = $msg;
						}
					}
				}
			}
			else if (!$other)
			{
				$msg = $arcade->root_key($file) . ' - <span style="color: black;">' . $user->lang['NOT_FOUND'] . '</span>';

				if (!in_array($msg, $error))
				{
					$error[] = $msg;
				}
			}
		}
	}

	function path_replace($mode, $path, &$file)
	{
		$search = '';

		switch ($mode)
		{
			case 'lang':
				$search = '{LANG_PATH}';
			break;

			case 'template':
				$search = '{TEMP_PATH}';
			break;

			case 'theme':
				$search = '{THEME_PATH}';
			break;
		}

		if ($search !== '')
		{
			$file = str_replace($search, $path, $file);
		}
	}

	function u_module($params = false, $is_amp = true, $session_id = false)
	{
		global $phpbb_root_path;
		return append_sid($phpbb_root_path . $this->module_url, $params, $is_amp, $session_id);
	}
}

?>